@@ -13,27 +13,30 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import shlex
+
 from neutronclient.common import exceptions as n_exc
 from oslo_log import log as logging
 from oslo_utils import strutils
 from oslo_utils import uuidutils
 import pecan
+import six
 
 from zun.api.controllers import base
 from zun.api.controllers import link
 from zun.api.controllers.v1 import collection
 from zun.api.controllers.v1.schemas import containers as schema
 from zun.api.controllers.v1.views import containers_view as view
-from zun.api.controllers import versions
 from zun.api import utils as api_utils
+from zun.api import validation
 from zun.common import consts
 from zun.common import context as zun_context
 from zun.common import exception
 from zun.common.i18n import _
 from zun.common import name_generator
+from zun.common.policies import container as policies
 from zun.common import policy
 from zun.common import utils
-from zun.common import validation
 import zun.conf
 from zun.network import model as network_model
 from zun.network import neutron
@@ -43,7 +46,6 @@ from zun.volume import cinder_api as cinder
 
 CONF = zun.conf.CONF
 LOG = logging.getLogger(__name__)
-NETWORK_ATTACH_EXTERNAL = 'network:attach_external_network'
 
 
 def check_policy_on_container(container, action):
@@ -68,13 +70,91 @@ class ContainerCollection(collection.Collection):
     @staticmethod
     def convert_with_links(rpc_containers, limit, url=None,
                            expand=False, **kwargs):
+        context = pecan.request.context
         collection = ContainerCollection()
         collection.containers = \
-            [view.format_container(url, p) for p in rpc_containers]
+            [view.format_container(context, url, p.as_dict())
+             for p in rpc_containers]
         collection.next = collection.get_next(limit, url=url, **kwargs)
         return collection
 
 
+class ContainersActionsController(base.Controller):
+    """Controller for Container Actions."""
+
+    def __init__(self):
+        super(ContainersActionsController, self).__init__()
+        self._action_keys = ['action', 'container_uuid', 'request_id',
+                             'user_id', 'project_id', 'start_time',
+                             'message']
+        self._event_keys = ['event', 'start_time', 'finish_time', 'result',
+                            'traceback']
+
+    def _format_action(self, action_raw):
+        action = {}
+        action_dict = action_raw.as_dict()
+        for key in self._action_keys:
+            action[key] = action_dict.get(key)
+        return action
+
+    def _format_event(self, event_raw, show_traceback=False):
+        event = {}
+        event_dict = event_raw.as_dict()
+        for key in self._event_keys:
+            # By default, non-admins are not allowed to see traceback details.
+            if key == 'traceback' and not show_traceback:
+                event['traceback'] = None
+                continue
+            event[key] = event_dict.get(key)
+        return event
+
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def get_all(self, container_ident, **kwargs):
+        """Retrieve a list of container actions."""
+        context = pecan.request.context
+        policy.enforce(context, "container:actions",
+                       action="container:actions")
+        container = utils.get_container(container_ident)
+        actions_raw = objects.ContainerAction.get_by_container_uuid(
+            context, container.uuid)
+        actions = [self._format_action(action) for action in actions_raw]
+
+        return {"containerActions": actions}
+
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def get_one(self, container_ident, request_ident, **kwargs):
+        """Retrieve information about the action."""
+
+        context = pecan.request.context
+        policy.enforce(context, "container:actions",
+                       action="container:actions")
+        container = utils.get_container(container_ident)
+        action = objects.ContainerAction.get_by_request_id(
+            context, container.uuid, request_ident)
+
+        if action is None:
+            raise exception.ResourceNotFound(name="Action", id=request_ident)
+
+        action_id = action.id
+        if CONF.database.backend == 'etcd':
+            # etcd using action.uuid get the unique action instead of action.id
+            action_id = action.uuid
+
+        action = self._format_action(action)
+        show_traceback = False
+        if policy.enforce(context, "container:action:events",
+                          do_raise=False, action="container:action:events"):
+            show_traceback = True
+
+        events_raw = objects.ContainerActionEvent.get_by_action(context,
+                                                                action_id)
+        action['events'] = [self._format_event(evt, show_traceback)
+                            for evt in events_raw]
+        return action
+
+
 class ContainersController(base.Controller):
     """Controller for Containers."""
 
@@ -82,6 +162,7 @@ class ContainersController(base.Controller):
         'start': ['POST'],
         'stop': ['POST'],
         'reboot': ['POST'],
+        'rebuild': ['POST'],
         'pause': ['POST'],
         'unpause': ['POST'],
         'logs': ['GET'],
@@ -91,6 +172,7 @@ class ContainersController(base.Controller):
         'rename': ['POST'],
         'attach': ['GET'],
         'resize': ['POST'],
+        'resize_container': ['POST'],
         'top': ['GET'],
         'get_archive': ['GET'],
         'put_archive': ['POST'],
@@ -99,9 +181,12 @@ class ContainersController(base.Controller):
         'add_security_group': ['POST'],
         'network_detach': ['POST'],
         'network_attach': ['POST'],
+        'network_list': ['GET'],
         'remove_security_group': ['POST']
     }
 
+    container_actions = ContainersActionsController()
+
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     def get_all(self, **kwargs):
@@ -119,27 +204,39 @@ class ContainersController(base.Controller):
             policy.enforce(context, "container:get_all_all_projects",
                            action="container:get_all_all_projects")
             context.all_projects = True
-        limit = api_utils.validate_limit(kwargs.get('limit'))
-        sort_dir = api_utils.validate_sort_dir(kwargs.get('sort_dir', 'asc'))
-        sort_key = kwargs.get('sort_key', 'id')
-        resource_url = kwargs.get('resource_url')
-        expand = kwargs.get('expand')
+        kwargs.pop('all_projects', None)
+        limit = api_utils.validate_limit(kwargs.pop('limit', None))
+        sort_dir = api_utils.validate_sort_dir(kwargs.pop('sort_dir', 'asc'))
+        sort_key = kwargs.pop('sort_key', 'id')
+        resource_url = kwargs.pop('resource_url', None)
+        expand = kwargs.pop('expand', None)
 
-        filters = None
+        container_allowed_filters = ['name', 'image', 'project_id', 'user_id',
+                                     'memory', 'host', 'task_state', 'status',
+                                     'auto_remove']
+        filters = {}
+        for filter_key in container_allowed_filters:
+            if filter_key in kwargs:
+                policy_action = policies.CONTAINER % ('get_one:' + filter_key)
+                context.can(policy_action, might_not_exist=True)
+                filter_value = kwargs.pop(filter_key)
+                filters[filter_key] = filter_value
         marker_obj = None
-        marker = kwargs.get('marker')
+        marker = kwargs.pop('marker', None)
         if marker:
             marker_obj = objects.Container.get_by_uuid(context,
                                                        marker)
+        if kwargs:
+            unknown_params = [str(k) for k in kwargs]
+            msg = _("Unknown parameters: %s") % ", ".join(unknown_params)
+            raise exception.InvalidValue(msg)
+
         containers = objects.Container.list(context,
                                             limit,
                                             marker_obj,
                                             sort_key,
                                             sort_dir,
                                             filters=filters)
-        if not context.is_admin:
-            for container in containers:
-                del container.host
         return ContainerCollection.convert_with_links(containers, limit,
                                                       url=resource_url,
                                                       expand=expand,
@@ -160,15 +257,15 @@ class ContainersController(base.Controller):
             context.all_projects = True
         container = utils.get_container(container_ident)
         check_policy_on_container(container.as_dict(), "container:get_one")
-        compute_api = pecan.request.compute_api
-        try:
-            container = compute_api.container_show(context, container)
-        except exception.ContainerHostNotUp:
-            raise exception.ServerNotUsable
+        if container.host:
+            compute_api = pecan.request.compute_api
+            try:
+                container = compute_api.container_show(context, container)
+            except exception.ContainerHostNotUp:
+                raise exception.ServerNotUsable
 
-        if not context.is_admin:
-            del container.host
-        return view.format_container(pecan.request.host_url, container)
+        return view.format_container(context, pecan.request.host_url,
+                                     container.as_dict())
 
     def _generate_name_for_container(self):
         """Generate a random name like: zeta-22-container."""
@@ -176,12 +273,34 @@ class ContainersController(base.Controller):
         name = name_gen.generate()
         return name + '-container'
 
+    @base.Controller.api_version("1.1", "1.19")
+    @pecan.expose('json')
+    @api_utils.enforce_content_types(['application/json'])
+    @exception.wrap_pecan_controller_exception
+    @validation.validate_query_param(pecan.request, schema.query_param_create)
+    @validation.validated(schema.legacy_container_create)
+    def post(self, run=False, **container_dict):
+        # NOTE(hongbin): We convert the representation of 'command' from
+        # string to list. For example:
+        # '"nginx" "-g" "daemon off;"' -> ["nginx", "-g", "daemon off;"]
+        command = container_dict.pop('command', None)
+        if command is not None:
+            if isinstance(command, six.string_types):
+                command = shlex.split(command)
+            container_dict['command'] = command
+
+        return self._do_post(run, **container_dict)
+
+    @base.Controller.api_version("1.20")  # noqa
     @pecan.expose('json')
     @api_utils.enforce_content_types(['application/json'])
     @exception.wrap_pecan_controller_exception
     @validation.validate_query_param(pecan.request, schema.query_param_create)
     @validation.validated(schema.container_create)
     def post(self, run=False, **container_dict):
+        return self._do_post(run, **container_dict)
+
+    def _do_post(self, run=False, **container_dict):
         """Create or run a new container.
 
         :param run: if true, starts the container
@@ -211,41 +330,25 @@ class ContainersController(base.Controller):
 
         auto_remove = container_dict.pop('auto_remove', None)
         if auto_remove is not None:
-            req_version = pecan.request.version
-            min_version = versions.Version('', '', '', '1.3')
-            if req_version >= min_version:
-                try:
-                    container_dict['auto_remove'] = strutils.bool_from_string(
-                        auto_remove, strict=True)
-                except ValueError:
-                    raise exception.InvalidValue(_('Auto_remove values are: '
-                                                   'true, false, True, False'))
-            else:
-                raise exception.InvalidParamInVersion(param='auto_remove',
-                                                      req_version=req_version,
-                                                      min_version=min_version)
+            api_utils.version_check('auto_remove', '1.3')
+            try:
+                container_dict['auto_remove'] = strutils.bool_from_string(
+                    auto_remove, strict=True)
+            except ValueError:
+                raise exception.InvalidValue(_('Auto_remove values are: '
+                                               'true, false, True, False'))
 
         runtime = container_dict.pop('runtime', None)
         if runtime is not None:
-            req_version = pecan.request.version
-            min_version = versions.Version('', '', '', '1.5')
-            if req_version >= min_version:
-                container_dict['runtime'] = runtime
-            else:
-                raise exception.InvalidParamInVersion(param='runtime',
-                                                      req_version=req_version,
-                                                      min_version=min_version)
+            api_utils.version_check('runtime', '1.5')
+            policy.enforce(context, "container:create:runtime",
+                           action="container:create:runtime")
+            container_dict['runtime'] = runtime
 
         hostname = container_dict.pop('hostname', None)
         if hostname is not None:
-            req_version = pecan.request.version
-            min_version = versions.Version('', '', '', '1.9')
-            if req_version >= min_version:
-                container_dict['hostname'] = hostname
-            else:
-                raise exception.InvalidParamInVersion(param='hostname',
-                                                      req_version=req_version,
-                                                      min_version=min_version)
+            api_utils.version_check('hostname', '1.9')
+            container_dict['hostname'] = hostname
 
         nets = container_dict.get('nets', [])
         requested_networks = utils.build_requested_networks(context, nets)
@@ -254,12 +357,7 @@ class ContainersController(base.Controller):
 
         mounts = container_dict.pop('mounts', [])
         if mounts:
-            req_version = pecan.request.version
-            min_version = versions.Version('', '', '', '1.11')
-            if req_version < min_version:
-                raise exception.InvalidParamInVersion(param='mounts',
-                                                      req_version=req_version,
-                                                      min_version=min_version)
+            api_utils.version_check('mounts', '1.11')
 
         requested_volumes = self._build_requested_volumes(context, mounts)
 
@@ -273,9 +371,7 @@ class ContainersController(base.Controller):
         name = container_dict.get('name') or \
             self._generate_name_for_container()
         container_dict['name'] = name
-        if container_dict.get('memory'):
-            container_dict['memory'] = \
-                str(container_dict['memory']) + 'M'
+        self._set_default_resource_limit(container_dict)
         if container_dict.get('restart_policy'):
             utils.check_for_restart_policy(container_dict)
 
@@ -283,6 +379,8 @@ class ContainersController(base.Controller):
         extra_spec = {}
         extra_spec['hints'] = container_dict.get('hints', None)
         extra_spec['pci_requests'] = pci_req
+        extra_spec['availability_zone'] = container_dict.get(
+            'availability_zone')
         new_container = objects.Container(context, **container_dict)
         new_container.create(context)
 
@@ -298,7 +396,17 @@ class ContainersController(base.Controller):
         pecan.response.location = link.build_url('containers',
                                                  new_container.uuid)
         pecan.response.status = 202
-        return view.format_container(pecan.request.host_url, new_container)
+        return view.format_container(context, pecan.request.host_url,
+                                     new_container.as_dict())
+
+    def _set_default_resource_limit(self, container_dict):
+        # NOTE(kiennt): Default disk size will be set later.
+        container_dict['disk'] = container_dict.get('disk')
+        container_dict['memory'] = container_dict.get(
+            'memory', CONF.default_memory)
+        container_dict['memory'] = str(container_dict['memory'])
+        container_dict['cpu'] = container_dict.get(
+            'cpu', CONF.default_cpu)
 
     def _create_pci_requests_for_sriov_ports(self, context,
                                              requested_networks):
@@ -417,6 +525,7 @@ class ContainersController(base.Controller):
             else:
                 raise
 
+    @base.Controller.api_version("1.1", "1.14")
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     @validation.validated(schema.add_security_group)
@@ -444,6 +553,7 @@ class ContainersController(base.Controller):
                                        security_group_id)
         pecan.response.status = 202
 
+    @base.Controller.api_version("1.1", "1.14")
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     @validation.validated(schema.remove_security_group)
@@ -482,14 +592,18 @@ class ContainersController(base.Controller):
         check_policy_on_container(container.as_dict(), "container:update")
         utils.validate_container_state(container, 'update')
         if 'memory' in patch:
-            patch['memory'] = str(patch['memory']) + 'M'
+            patch['memory'] = str(patch['memory'])
         if 'cpu' in patch:
             patch['cpu'] = float(patch['cpu'])
+        if 'name' in patch:
+            patch['name'] = str(patch['name'])
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         container = compute_api.container_update(context, container, patch)
-        return view.format_container(pecan.request.host_url, container)
+        return view.format_container(context, pecan.request.host_url,
+                                     container.as_dict())
 
+    @base.Controller.api_version("1.1", "1.13")
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     @validation.validate_query_param(pecan.request, schema.query_param_rename)
@@ -497,7 +611,7 @@ class ContainersController(base.Controller):
         """Rename an existing container.
 
         :param container_ident: UUID or Name of a container.
-        :param patch: a json PATCH document to apply to this container.
+        :param name: a new name for this container.
         """
         container = utils.get_container(container_ident)
         check_policy_on_container(container.as_dict(), "container:rename")
@@ -507,7 +621,33 @@ class ContainersController(base.Controller):
         container.name = name
         context = pecan.request.context
         container.save(context)
-        return view.format_container(pecan.request.host_url, container)
+        return view.format_container(context, pecan.request.host_url,
+                                     container.as_dict())
+
+    @base.Controller.api_version("1.19")
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    @validation.validated(schema.container_update)
+    def resize_container(self, container_ident, **kwargs):
+        """Resize an existing container.
+
+        :param container_ident: UUID or name of a container.
+        :param kwargs: cpu/memory to be updated.
+        """
+        container = utils.get_container(container_ident)
+        check_policy_on_container(container.as_dict(),
+                                  "container:resize_container")
+        utils.validate_container_state(container, 'resize_container')
+        if 'memory' in kwargs:
+            kwargs['memory'] = str(kwargs['memory'])
+        if 'cpu' in kwargs:
+            kwargs['cpu'] = float(kwargs['cpu'])
+        context = pecan.request.context
+        compute_api = pecan.request.compute_api
+        compute_api.resize_container(context, container, kwargs)
+        pecan.response.status = 202
+        return view.format_container(context, pecan.request.host_url,
+                                     container.as_dict())
 
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
@@ -542,37 +682,50 @@ class ContainersController(base.Controller):
         if not force and not stop:
             utils.validate_container_state(container, 'delete')
         elif force and not stop:
-            req_version = pecan.request.version
-            min_version = versions.Version('', '', '', '1.7')
-            if req_version >= min_version:
-                policy.enforce(context, "container:delete_force",
-                               action="container:delete_force")
-                utils.validate_container_state(container, 'delete_force')
-            else:
-                raise exception.InvalidParamInVersion(param='force',
-                                                      req_version=req_version,
-                                                      min_version=min_version)
+            api_utils.version_check('force', '1.7')
+            policy.enforce(context, "container:delete_force",
+                           action="container:delete_force")
+            utils.validate_container_state(container, 'delete_force')
         elif stop:
-            req_version = pecan.request.version
-            min_version = versions.Version('', '', '', '1.12')
-            if req_version >= min_version:
-                check_policy_on_container(container.as_dict(),
-                                          "container:stop")
-                utils.validate_container_state(container,
-                                               'delete_after_stop')
-                if container.status == consts.RUNNING:
-                    LOG.debug('Calling compute.container_stop with %s '
-                              'before delete',
-                              container.uuid)
-                    compute_api.container_stop(context, container, 10)
-            else:
-                raise exception.InvalidParamInVersion(param='stop',
-                                                      req_version=req_version,
-                                                      min_version=min_version)
+            api_utils.version_check('stop', '1.12')
+            check_policy_on_container(container.as_dict(),
+                                      "container:stop")
+            utils.validate_container_state(container,
+                                           'delete_after_stop')
+            if container.status == consts.RUNNING:
+                LOG.debug('Calling compute.container_stop with %s '
+                          'before delete',
+                          container.uuid)
+                compute_api.container_stop(context, container, 10)
         container.status = consts.DELETING
-        compute_api.container_delete(context, container, force)
+        if container.host:
+            compute_api.container_delete(context, container, force)
+        else:
+            container.destroy(context)
         pecan.response.status = 204
 
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def rebuild(self, container_ident, **kwargs):
+        """Rebuild container.
+
+        :param container_ident: UUID or Name of a container.
+        """
+        container = utils.get_container(container_ident)
+        check_policy_on_container(container.as_dict(), "container:rebuild")
+        utils.validate_container_state(container, 'rebuild')
+        if kwargs.get('image'):
+            container.image = kwargs.get('image')
+        if kwargs.get('image_driver'):
+            utils.validate_image_driver(kwargs.get('image_driver'))
+            container.image_driver = kwargs.get('image_driver')
+        LOG.debug('Calling compute.container_rebuild with %s',
+                  container.uuid)
+        context = pecan.request.context
+        compute_api = pecan.request.compute_api
+        compute_api.container_rebuild(context, container)
+        pecan.response.status = 202
+
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     def start(self, container_ident, **kwargs):
@@ -622,6 +775,8 @@ class ContainersController(base.Controller):
         LOG.debug('Calling compute.container_reboot with %s',
                   container.uuid)
         context = pecan.request.context
+        container.status = consts.RESTARTING
+        container.save(context)
         compute_api = pecan.request.compute_api
         compute_api.container_reboot(context, container, timeout)
         pecan.response.status = 202
@@ -669,7 +824,7 @@ class ContainersController(base.Controller):
 
         :param container_ident: UUID or Name of a container.
         :param stdout: Get standard output if True.
-        :param sterr: Get standard error if True.
+        :param stderr: Get standard error if True.
         :param timestamps: Show timestamps.
         :param tail: Number of lines to show from the end of the logs.
                      (default: get all logs)
@@ -910,8 +1065,13 @@ class ContainersController(base.Controller):
         context = pecan.request.context
         compute_api = pecan.request.compute_api
         neutron_api = neutron.NeutronAPI(context)
-        neutron_net = neutron_api.get_neutron_network(kwargs.get('network'))
-        compute_api.network_detach(context, container, neutron_net['id'])
+        if kwargs.get('port'):
+            port = neutron_api.get_neutron_port(kwargs['port'])
+            net_id = port['network_id']
+        else:
+            network = neutron_api.get_neutron_network(kwargs.get('network'))
+            net_id = network['id']
+        compute_api.network_detach(context, container, net_id)
         pecan.response.status = 202
 
     @base.Controller.api_version("1.8")
@@ -928,6 +1088,62 @@ class ContainersController(base.Controller):
                                   "container:network_attach")
         context = pecan.request.context
         compute_api = pecan.request.compute_api
-        neutron_api = neutron.NeutronAPI(context)
-        neutron_net = neutron_api.get_neutron_network(kwargs.get('network'))
-        compute_api.network_attach(context, container, neutron_net['id'])
+        requested_networks = utils.build_requested_networks(context, [kwargs])
+        compute_api.network_attach(context, container, requested_networks[0])
+
+    @base.Controller.api_version("1.13", "1.17")
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def network_list(self, container_ident):
+        """Retrieve a list of networks of the container.
+
+        :param container_ident: UUID or Name of a container.
+        """
+        container = utils.get_container(container_ident)
+        container_networks = self._get_container_networks_legacy(container)
+        return {'networks': container_networks}
+
+    def _get_container_networks_legacy(self, container):
+        container_networks = []
+        for net_id, net_infos in container.addresses.items():
+            for net_info in net_infos:
+                container_networks.append({
+                    'net_id': net_id,
+                    'subnet_id': net_info.get("subnet_id"),
+                    'port_id': net_info.get("port"),
+                    'version': net_info.get("version"),
+                    'ip_address': net_info.get("addr")
+                })
+        return container_networks
+
+    @base.Controller.api_version("1.18")  # noqa
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def network_list(self, container_ident):
+        """Retrieve a list of networks of the container.
+
+        :param container_ident: UUID or Name of a container.
+        """
+        container = utils.get_container(container_ident)
+        container_networks = self._get_container_networks(container)
+        return {'networks': container_networks}
+
+    def _get_container_networks(self, container):
+        container_networks = []
+        for net_id, net_infos in container.addresses.items():
+            addresses = {}
+            for net_info in net_infos:
+                port_id = net_info["port"]
+                addresses.setdefault(port_id, [])
+                addresses[port_id].append({
+                    'subnet_id': net_info.get("subnet_id"),
+                    'version': net_info.get("version"),
+                    'ip_address': net_info.get("addr")
+                })
+            for port_id, fixed_ips in addresses.items():
+                container_networks.append({
+                    'net_id': net_id,
+                    'port_id': port_id,
+                    'fixed_ips': fixed_ips,
+                })
+        return container_networks
