@@ -20,16 +20,29 @@ from zun.api.controllers.v1 import collection
 from zun.api.controllers.v1.schemas import images as schema
 from zun.api.controllers.v1.views import images_view as view
 from zun.api import utils as api_utils
+from zun.api import validation
 from zun.common import exception
 from zun.common.i18n import _
 from zun.common import policy
 from zun.common import utils
-from zun.common import validation
 from zun import objects
 
 LOG = logging.getLogger(__name__)
 
 
+def check_policy_on_image(image, action):
+    context = pecan.request.context
+    policy.enforce(context, action, image, action=action)
+
+
+def _get_host(host_ident):
+    try:
+        return api_utils.get_resource('ComputeNode', host_ident)
+    except exception.ComputeNodeNotFound:
+        msg = _("The host %s does not exist.") % host_ident
+        raise exception.InvalidValue(msg)
+
+
 class ImageCollection(collection.Collection):
     """API representation of a collection of images."""
 
@@ -60,6 +73,17 @@ class ImagesController(base.Controller):
         'search': ['GET']
     }
 
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    @validation.validate_query_param(pecan.request, schema.query_param_delete)
+    def delete(self, image_id, **kwargs):
+        context = pecan.request.context
+        policy.enforce(context, "image:delete", action="image:delete")
+        host = _get_host(kwargs.pop('host'))
+        image = utils.get_image(image_id)
+        return pecan.request.compute_api.image_delete(context, image,
+                                                      host.hostname)
+
     @pecan.expose('json')
     @exception.wrap_pecan_controller_exception
     def get_all(self, **kwargs):
@@ -69,6 +93,17 @@ class ImagesController(base.Controller):
                        action="image:get_all")
         return self._get_images_collection(**kwargs)
 
+    @pecan.expose('json')
+    @exception.wrap_pecan_controller_exception
+    def get_one(self, image_id):
+        """Retrieve information about the given image.
+
+        :param image_id: UUID of a image.
+        """
+        image = utils.get_image(image_id)
+        check_policy_on_image(image.as_dict(), "image:get_one")
+        return view.format_image(pecan.request.host_url, image)
+
     def _get_images_collection(self, **kwargs):
         context = pecan.request.context
         limit = api_utils.validate_limit(kwargs.get('limit'))
@@ -100,11 +135,12 @@ class ImagesController(base.Controller):
     def post(self, **image_dict):
         """Create a new image.
 
-        :param image: an image within the request body.
+        :param image_dict: an image within the request body.
         """
         context = pecan.request.context
         policy.enforce(context, "image:pull",
                        action="image:pull")
+        host = _get_host(image_dict.pop('host'))
         image_dict['project_id'] = context.project_id
         image_dict['user_id'] = context.user_id
         repo_tag = image_dict.get('repo')
@@ -112,7 +148,7 @@ class ImagesController(base.Controller):
             repo_tag)
         new_image = objects.Image(context, **image_dict)
         new_image.pull(context)
-        pecan.request.compute_api.image_pull(context, new_image)
+        pecan.request.compute_api.image_pull(context, new_image, host.hostname)
         # Set the HTTP Location Header
         pecan.response.location = link.build_url('images', new_image.uuid)
         pecan.response.status = 202
@@ -122,6 +158,12 @@ class ImagesController(base.Controller):
     @exception.wrap_pecan_controller_exception
     @validation.validate_query_param(pecan.request, schema.query_param_search)
     def search(self, image, image_driver=None, exact_match=False):
+        """Search a specific image
+
+        :param image:  Name of the image.
+        :param image_driver: Name of the image driver (glance, docker).
+        :param exact_match: if True, exact match the image name.
+        """
         context = pecan.request.context
         policy.enforce(context, "image:search",
                        action="image:search")
@@ -129,9 +171,9 @@ class ImagesController(base.Controller):
         try:
             exact_match = strutils.bool_from_string(exact_match, strict=True)
         except ValueError:
-            msg = _("Valid exact_match values are true,"
-                    " false, 0, 1, yes and no")
-            raise exception.InvalidValue(msg)
+            bools = ', '.join(strutils.TRUE_STRINGS + strutils.FALSE_STRINGS)
+            raise exception.InvalidValue(_('Valid exact_match values are: %s')
+                                         % bools)
         # Valiadtion accepts 'None' so need to convert it to None
         if image_driver:
             image_driver = api_utils.string_or_none(image_driver)
