@@ -12,9 +12,9 @@
 
 import copy
 
-from zun.common.validation import parameter_types
+from zun.api.controllers.v1.schemas import parameter_types
 
-_container_properties = {
+_legacy_container_properties = {
     'name': parameter_types.container_name,
     'image': parameter_types.image_name,
     'command': parameter_types.command,
@@ -30,9 +30,25 @@ _container_properties = {
     'image_driver': parameter_types.image_driver,
     'security_groups': parameter_types.security_groups,
     'hints': parameter_types.hints,
-    'nets': parameter_types.nets
+    'mounts': parameter_types.mounts,
+    'nets': parameter_types.nets,
+    'runtime': parameter_types.runtime,
+    'hostname': parameter_types.hostname,
+    'disk': parameter_types.disk,
+    'availability_zone': parameter_types.availability_zone,
+    'auto_heal': parameter_types.boolean,
 }
 
+legacy_container_create = {
+    'type': 'object',
+    'properties': _legacy_container_properties,
+    'required': ['image'],
+    'additionalProperties': False
+}
+
+_container_properties = copy.deepcopy(_legacy_container_properties)
+_container_properties['command'] = parameter_types.command_list
+
 container_create = {
     'type': 'object',
     'properties': _container_properties,
@@ -59,6 +75,7 @@ query_param_create = {
 _container_update_properties = {
     'cpu': parameter_types.cpu,
     'memory': parameter_types.memory,
+    'name': parameter_types.container_name
 }
 
 container_update = {
@@ -71,7 +88,8 @@ query_param_delete = {
     'type': 'object',
     'properties': {
         'force': parameter_types.boolean_extended,
-        'all_tenants': parameter_types.boolean_extended
+        'all_projects': parameter_types.boolean_extended,
+        'stop': parameter_types.boolean_extended
     },
     'additionalProperties': False
 }
@@ -158,13 +176,66 @@ add_security_group = {
             'type': 'string',
             'minLength': 1,
             'maxLength': 255
-        },
-        'uuid': {
-            'type': 'string',
-            'minLength': 2,
-            'maxLength': 255,
-            'pattern': '[a-zA-Z0-9][a-zA-Z0-9_.-]'
         }
     },
     'additionalProperties': False
 }
+
+remove_security_group = copy.deepcopy(add_security_group)
+
+network_detach = {
+    'type': 'object',
+    'properties': {
+        'network': {
+            'type': 'string',
+            'minLength': 1,
+            'maxLength': 255,
+        },
+        'port': {
+            'type': 'string',
+            'minLength': 1,
+            'maxLength': 255,
+        }
+    },
+    'oneOf': [
+        {
+            'required': ['network']
+        },
+        {
+            'required': ['port']
+        }
+    ],
+    'additionalProperties': False
+}
+
+network_attach = {
+    'type': 'object',
+    'properties': {
+        'network': {
+            'type': ['string'],
+            'minLength': 1,
+            'maxLength': 255,
+        },
+        'fixed_ip': {
+            'type': ['string'],
+            'oneOf': [
+                {'format': 'ipv4'},
+                {'format': 'ipv6'}
+            ]
+        },
+        'port': {
+            'type': ['string'],
+            'maxLength': 255,
+            'minLength': 1,
+        }
+    },
+    'additionalProperties': False,
+    'oneOf': [
+        {
+            'required': ['network']
+        },
+        {
+            'required': ['port']
+        }
+    ]
+}
