@@ -37,10 +37,26 @@ REST_API_VERSION_HISTORY = """REST API Version History:
     * 1.2 - Support user specify pre created networks
     * 1.3 - Add auto_remove to container
     * 1.4 - Support list all container host and show a container host
+    * 1.5 - Add runtime to container
+    * 1.6 - Support detach network from a container
+    * 1.7 - Disallow non-admin users to force delete containers
+    * 1.8 - Support attach a network to a container
+    * 1.9 - Add support set container's hostname
+    * 1.10 - Make delete container async
+    * 1.11 - Add mounts to container create
+    * 1.12 - Add support to stop container before delete
+    * 1.13 - Add support for listing networks of a container
+    * 1.14 - Add support to rename the container from update api
+    * 1.15 - Remove add_security_group and remove_security_group
+    * 1.16 - Modify restart_policy to capsule spec content
+    * 1.17 - Add support for detaching ports
+    * 1.18 - Modify the response of network list
+    * 1.19 - Intoduce container resize API
+    * 1.20 - Convert type of 'command' from string to list
 """
 
 BASE_VER = '1.1'
-CURRENT_MAX_VER = '1.4'
+CURRENT_MAX_VER = '1.20'
 
 
 class Version(object):
