@@ -58,11 +58,8 @@ class ContextHook(hooks.PecanHook):
         roles = headers.get('X-Roles', '').split(',')
         auth_token_info = state.request.environ.get('keystone.token_info')
 
-        auth_url = CONF.keystone_authtoken.auth_uri
-
         state.request.context = context.make_context(
             auth_token=auth_token,
-            auth_url=auth_url,
             auth_token_info=auth_token_info,
             user_name=user_name,
             user_id=user_id,
