@@ -19,12 +19,14 @@
 import eventlet
 import functools
 import inspect
+import json
 import mimetypes
 
 from oslo_concurrency import lockutils
 from oslo_concurrency import processutils
 from oslo_context import context as common_context
 from oslo_log import log as logging
+from oslo_utils import excutils
 from oslo_utils import strutils
 import pecan
 import six
@@ -43,19 +45,22 @@ CONF = zun.conf.CONF
 LOG = logging.getLogger(__name__)
 NETWORK_ATTACH_EXTERNAL = 'network:attach_external_network'
 
-synchronized = lockutils.synchronized_with_prefix('zun-')
+synchronized = lockutils.synchronized_with_prefix(consts.NAME_PREFIX)
 
 VALID_STATES = {
     'commit': [consts.RUNNING, consts.STOPPED, consts.PAUSED],
-    'delete': [consts.CREATED, consts.ERROR, consts.STOPPED, consts.DELETED],
+    'delete': [consts.CREATED, consts.ERROR, consts.STOPPED, consts.DELETED,
+               consts.DEAD],
     'delete_force': [consts.CREATED, consts.CREATING, consts.ERROR,
                      consts.RUNNING, consts.STOPPED, consts.UNKNOWN,
-                     consts.DELETED],
+                     consts.DELETED, consts.DEAD, consts.RESTARTING,
+                     consts.REBUILDING],
     'delete_after_stop': [consts.RUNNING, consts.CREATED, consts.ERROR,
-                          consts.STOPPED, consts.DELETED],
+                          consts.STOPPED, consts.DELETED, consts.DEAD],
     'start': [consts.CREATED, consts.STOPPED, consts.ERROR],
     'stop': [consts.RUNNING],
     'reboot': [consts.CREATED, consts.RUNNING, consts.STOPPED, consts.ERROR],
+    'rebuild': [consts.CREATED, consts.RUNNING, consts.STOPPED, consts.ERROR],
     'pause': [consts.RUNNING],
     'unpause': [consts.PAUSED],
     'kill': [consts.RUNNING],
@@ -75,7 +80,9 @@ VALID_STATES = {
     'add_security_group': [consts.CREATED, consts.RUNNING, consts.STOPPED,
                            consts.PAUSED],
     'remove_security_group': [consts.CREATED, consts.RUNNING, consts.STOPPED,
-                              consts.PAUSED]
+                              consts.PAUSED],
+    'resize_container': [consts.CREATED, consts.RUNNING, consts.STOPPED,
+                         consts.PAUSED]
 }
 
 VALID_CONTAINER_FILED = {
@@ -109,6 +116,13 @@ def validate_container_state(container, action):
             actual_state=container.status)
 
 
+def validate_image_driver(image_driver):
+    if image_driver not in CONF.image_driver_list:
+        detail = _("Invalid input for image_driver, "
+                   "it should be within the image drivers list")
+        raise exception.ValidationError(detail=detail)
+
+
 def safe_rstrip(value, chars=None):
     """Removes trailing characters from a string if that does not make it empty
 
@@ -145,11 +159,16 @@ def allow_all_content_types(f):
     return _do_allow_certain_content_types(f, mimetypes.types_map.values())
 
 
-def parse_image_name(image):
+def parse_image_name(image, driver=None):
     image_parts = image.split(':', 1)
 
     image_repo = image_parts[0]
-    image_tag = 'latest'
+    if driver is None:
+        driver = CONF.default_image_driver
+    if driver == 'glance':
+        image_tag = ''
+    else:
+        image_tag = 'latest'
 
     if len(image_parts) > 1:
         image_tag = image_parts[1]
@@ -218,7 +237,7 @@ def check_container_id(function):
 
 def get_image_pull_policy(image_pull_policy, image_tag):
     if not image_pull_policy:
-        if image_tag == 'latest':
+        if image_tag == 'latest' or not image_tag:
             image_pull_policy = 'always'
         else:
             image_pull_policy = 'ifnotpresent'
@@ -283,7 +302,7 @@ def parse_floating_cpu(spec):
 
 
 def get_security_group_ids(context, security_groups, **kwargs):
-    if security_groups is None:
+    if not security_groups:
         return None
     else:
         neutron = clients.OpenStackClients(context).neutron()
@@ -310,6 +329,10 @@ def custom_execute(*cmd, **kwargs):
                                      error=six.text_type(e))
 
 
+def get_root_helper():
+    return 'sudo zun-rootwrap %s' % CONF.rootwrap_config
+
+
 @privileged.default.entrypoint
 def execute_root(*cmd, **kwargs):
     # NOTE(kiennt): Set run_as_root=False because if it is set to True, the
@@ -333,22 +356,32 @@ def execute(*cmd, **kwargs):
 
 def check_capsule_template(tpl):
     # TODO(kevinz): add volume spec check
-    kind_field = tpl.get('kind')
+    tpl_json = tpl
+    if isinstance(tpl, six.string_types):
+        try:
+            tpl_json = json.loads(tpl)
+        except Exception as e:
+            raise exception.FailedParseStringToJson(e)
+
+    kind_field = tpl_json.get('kind')
     if kind_field not in ['capsule', 'Capsule']:
         raise exception.InvalidCapsuleTemplate("kind fields need to be "
                                                "set as capsule or Capsule")
-    # Align the Capsule restartPolicy with container restart_policy
-    if 'restartPolicy' in tpl.keys():
-        tpl['restartPolicy'] = \
-            VALID_CAPSULE_RESTART_POLICY[tpl['restartPolicy']]
-        tpl[VALID_CAPSULE_FIELD['restartPolicy']] = tpl.pop('restartPolicy')
 
-    spec_field = tpl.get('spec')
+    spec_field = tpl_json.get('spec')
     if spec_field is None:
         raise exception.InvalidCapsuleTemplate("No Spec found")
+    # Align the Capsule restartPolicy with container restart_policy
+    # Also change the template filed name from Kubernetes type to OpenStack
+    # type.
+    if 'restartPolicy' in spec_field.keys():
+        spec_field['restartPolicy'] = \
+            VALID_CAPSULE_RESTART_POLICY[spec_field['restartPolicy']]
+        spec_field[VALID_CAPSULE_FIELD['restartPolicy']] = \
+            spec_field.pop('restartPolicy')
     if spec_field.get('containers') is None:
         raise exception.InvalidCapsuleTemplate("No valid containers field")
-    return spec_field
+    return spec_field, tpl_json
 
 
 def capsule_get_container_spec(spec_field):
@@ -471,8 +504,7 @@ def build_requested_networks(context, nets):
                                        'router:external':
                                            network.get('router:external'),
                                        'shared': network.get('shared'),
-                                       'v4-fixed-ip': '',
-                                       'v6-fixed-ip': '',
+                                       'fixed_ip': '',
                                        'preserve_on_delete': True})
         elif net.get('network'):
             network = neutron_api.get_neutron_network(net['network'])
@@ -481,9 +513,9 @@ def build_requested_networks(context, nets):
                                        'router:external':
                                            network.get('router:external'),
                                        'shared': network.get('shared'),
-                                       'v4-fixed-ip':
-                                           net.get('v4-fixed-ip', ''),
-                                       'v6-fixed-ip':
+                                       'fixed_ip':
+                                           net.get('fixed_ip') or
+                                           net.get('v4-fixed-ip', '') or
                                            net.get('v6-fixed-ip', ''),
                                        'preserve_on_delete': False})
     if not requested_networks:
@@ -492,8 +524,7 @@ def build_requested_networks(context, nets):
         neutron_net = neutron_api.get_available_network()
         requested_networks.append({'network': neutron_net['id'],
                                    'port': '',
-                                   'v4-fixed-ip': '',
-                                   'v6-fixed-ip': '',
+                                   'fixed_ip': '',
                                    'preserve_on_delete': False})
 
     check_external_network_attach(context, requested_networks)
@@ -576,3 +607,21 @@ def wrap_container_event(prefix):
                 return function(self, context, *args, **kwargs)
         return decorated_function
     return helper
+
+
+def wrap_exception():
+    def helper(function):
+
+        @functools.wraps(function)
+        def decorated_function(self, context, container, *args, **kwargs):
+            try:
+                return function(self, context, container, *args, **kwargs)
+            except exception.DockerError as e:
+                with excutils.save_and_reraise_exception(reraise=False):
+                    LOG.error("Error occurred while calling Docker API: %s",
+                              six.text_type(e))
+            except Exception as e:
+                with excutils.save_and_reraise_exception(reraise=False):
+                    LOG.exception("Unexpected exception: %s", six.text_type(e))
+        return decorated_function
+    return helper
