@@ -13,8 +13,11 @@
 """Handles all requests relating to compute resources (e.g. containers,
 networking and storage of containers, and compute hosts on which they run)."""
 
+from oslo_log import log as logging
+
 from zun.common import consts
 from zun.common import exception
+from zun.common.i18n import _
 from zun.common import profiler
 from zun.compute import container_actions
 from zun.compute import rpcapi
@@ -22,7 +25,9 @@ import zun.conf
 from zun import objects
 from zun.scheduler import client as scheduler_client
 
+
 CONF = zun.conf.CONF
+LOG = logging.getLogger(__name__)
 
 
 @profiler.trace_cls("rpc")
@@ -41,15 +46,20 @@ class API(object):
     def container_create(self, context, new_container, extra_spec,
                          requested_networks, requested_volumes, run,
                          pci_requests=None):
-        host_state = None
         try:
             host_state = self._schedule_container(context, new_container,
                                                   extra_spec)
-        except Exception as exc:
+        except exception.NoValidHost:
             new_container.status = consts.ERROR
-            new_container.status_reason = str(exc)
+            new_container.status_reason = _(
+                "There are not enough hosts available.")
             new_container.save(context)
             return
+        except Exception:
+            new_container.status = consts.ERROR
+            new_container.status_reason = _("Unexpected exception occurred.")
+            new_container.save(context)
+            raise
 
         # NOTE(mkrai): Intent here is to check the existence of image
         # before proceeding to create container. If image is not found,
@@ -61,6 +71,10 @@ class API(object):
                     new_container.image_driver, True, host_state['host'])
                 if not images:
                     raise exception.ImageNotFound(image=new_container.image)
+                if len(images) > 1:
+                    raise exception.Conflict('Multiple images exist with same '
+                                             'name. Please use the container '
+                                             'uuid instead.')
             except Exception as e:
                 new_container.status = consts.ERROR
                 new_container.status_reason = str(e)
@@ -84,8 +98,13 @@ class API(object):
         self._record_action_start(context, container, container_actions.DELETE)
         return self.rpcapi.container_delete(context, container, *args)
 
-    def container_show(self, context, container, *args):
-        return self.rpcapi.container_show(context, container, *args)
+    def container_show(self, context, container):
+        return self.rpcapi.container_show(context, container)
+
+    def container_rebuild(self, context, container):
+        self._record_action_start(context, container,
+                                  container_actions.REBUILD)
+        return self.rpcapi.container_rebuild(context, container)
 
     def container_reboot(self, context, container, *args):
         self._record_action_start(context, container, container_actions.REBOOT)
@@ -114,7 +133,15 @@ class API(object):
                                           timestamps, tail, since)
 
     def container_exec(self, context, container, *args):
-        return self.rpcapi.container_exec(context, container, *args)
+        data = self.rpcapi.container_exec(context, container, *args)
+        token = data.pop('token', None)
+        exec_id = data.get('exec_id')
+        if token:
+            data['proxy_url'] = '%s?token=%s&uuid=%s&exec_id=%s' % (
+                CONF.websocket_proxy.base_url, token, container.uuid, exec_id)
+        else:
+            data['proxy_url'] = None
+        return data
 
     def container_exec_resize(self, context, container, *args):
         return self.rpcapi.container_exec_resize(context, container, *args)
@@ -126,8 +153,11 @@ class API(object):
     def container_update(self, context, container, *args):
         return self.rpcapi.container_update(context, container, *args)
 
-    def container_attach(self, context, container, *args):
-        return self.rpcapi.container_attach(context, container, *args)
+    def container_attach(self, context, container):
+        token = self.rpcapi.container_attach(context, container)
+        access_url = '%s?token=%s&uuid=%s' % (
+            CONF.websocket_proxy.base_url, token, container.uuid)
+        return access_url
 
     def container_resize(self, context, container, *args):
         return self.rpcapi.container_resize(context, container, *args)
@@ -158,30 +188,41 @@ class API(object):
         self._record_action_start(context, container, container_actions.COMMIT)
         return self.rpcapi.container_commit(context, container, *args)
 
-    def image_pull(self, context, image):
-        return self.rpcapi.image_pull(context, image)
+    def image_delete(self, context, image, host):
+        return self.rpcapi.image_delete(context, image, host)
+
+    def image_pull(self, context, image, host):
+        return self.rpcapi.image_pull(context, image, host)
 
     def image_search(self, context, image, image_driver, exact_match, *args):
         return self.rpcapi.image_search(context, image, image_driver,
                                         exact_match, *args)
 
-    def capsule_create(self, context, new_capsule, requested_networks=None,
-                       requested_volumes=None, extra_spec=None):
-        host_state = None
+    def capsule_create(self, context, new_capsule, requested_networks,
+                       requested_volumes, extra_spec):
         try:
             host_state = self._schedule_container(context, new_capsule,
                                                   extra_spec)
-        except Exception as exc:
-            new_capsule.status = consts.ERROR
-            new_capsule.status_reason = str(exc)
+        except exception.NoValidHost:
+            new_capsule.status = consts.FAILED
+            new_capsule.status_reason = _(
+                "There are not enough hosts available.")
             new_capsule.save(context)
             return
+        except Exception:
+            new_capsule.status = consts.FAILED
+            new_capsule.status_reason = _("Unexpected exception occurred.")
+            new_capsule.save(context)
+            raise
+        for container in new_capsule.containers:
+            self._record_action_start(context, container,
+                                      container_actions.CREATE)
         self.rpcapi.capsule_create(context, host_state['host'], new_capsule,
                                    requested_networks, requested_volumes,
                                    host_state['limits'])
 
-    def capsule_delete(self, context, capsule, *args):
-        return self.rpcapi.capsule_delete(context, capsule, *args)
+    def capsule_delete(self, context, capsule):
+        return self.rpcapi.capsule_delete(context, capsule)
 
     def network_detach(self, context, container, *args):
         self._record_action_start(context, container,
@@ -192,3 +233,9 @@ class API(object):
         self._record_action_start(context, container,
                                   container_actions.NETWORK_ATTACH)
         return self.rpcapi.network_attach(context, container, *args)
+
+    def network_create(self, context, network):
+        return self.rpcapi.network_create(context, network)
+
+    def resize_container(self, context, container, *args):
+        return self.rpcapi.resize_container(context, container, *args)
