@@ -47,13 +47,13 @@ class API(rpc_service.API):
         * 1.1 - Add image endpoints.
     """
 
-    def __init__(self, context=None, topic=None):
+    def __init__(self, transport=None, context=None, topic=None):
         if topic is None:
             zun.conf.CONF.import_opt(
                 'topic', 'zun.conf.compute', group='compute')
 
         super(API, self).__init__(
-            context, topic=zun.conf.CONF.compute.topic)
+            transport, context, topic=zun.conf.CONF.compute.topic)
 
     def container_create(self, context, host, container, limits,
                          requested_networks, requested_volumes, run,
@@ -75,6 +75,9 @@ class API(rpc_service.API):
         return self._call(container.host, 'container_show',
                           container=container)
 
+    def container_rebuild(self, context, container):
+        self._cast(container.host, 'container_rebuild', container=container)
+
     def container_reboot(self, context, container, timeout):
         self._cast(container.host, 'container_reboot', container=container,
                    timeout=timeout)
@@ -120,6 +123,10 @@ class API(rpc_service.API):
         return self._call(container.host, 'container_update',
                           container=container, patch=patch)
 
+    def resize_container(self, context, container, patch):
+        self._cast(container.host, 'resize_container',
+                   container=container, patch=patch)
+
     @check_container_host
     def container_attach(self, context, container):
         return self._call(container.host, 'container_attach',
@@ -163,11 +170,10 @@ class API(rpc_service.API):
         return self._cast(container.host, 'remove_security_group',
                           container=container, security_group=security_group)
 
-    def image_pull(self, context, image):
-        # NOTE(hongbin): Image API doesn't support multiple compute nodes
-        # scenario yet, so we temporarily set host to None and rpc will
-        # choose an arbitrary host.
-        host = None
+    def image_delete(self, context, image, host):
+        self._cast(host, 'image_delete', image=image)
+
+    def image_pull(self, context, image, host):
         self._cast(host, 'image_pull', image=image)
 
     def image_search(self, context, image, image_driver, exact_match,
@@ -189,9 +195,14 @@ class API(rpc_service.API):
                           capsule=capsule)
 
     def network_detach(self, context, container, network):
-        return self._call(container.host, 'network_detach',
-                          container=container, network=network)
+        self._cast(container.host, 'network_detach',
+                   container=container, network=network)
 
-    def network_attach(self, context, container, network):
-        return self._call(container.host, 'network_attach',
-                          container=container, network=network)
+    def network_attach(self, context, container, requested_network):
+        self._cast(container.host, 'network_attach',
+                   container=container,
+                   requested_network=requested_network)
+
+    def network_create(self, context, new_network):
+        host = None
+        return self._call(host, 'network_create', network=new_network)
