@@ -14,21 +14,25 @@
 import datetime
 import eventlet
 import functools
-import six
+import types
 
 from docker import errors
 from oslo_log import log as logging
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
+import six
 
 from zun.common import consts
 from zun.common import exception
 from zun.common.i18n import _
 from zun.common import utils
 from zun.common.utils import check_container_id
+from zun.compute import api as zun_compute
 import zun.conf
 from zun.container.docker import host
 from zun.container.docker import utils as docker_utils
 from zun.container import driver
+from zun.image import driver as img_driver
 from zun.network import network as zun_network
 from zun import objects
 from zun.volume import driver as vol_driver
@@ -43,6 +47,17 @@ def is_not_found(e):
     return '404' in str(e)
 
 
+def is_not_connected(e):
+    # Test the following exception:
+    #
+    #   500 Server Error: Internal Server Error ("container XXX is not
+    #   connected to the network XXX")
+    #
+    # Note(hongbin): Docker should response a 4xx instead of 500. This looks
+    # like a bug from docker side: https://github.com/moby/moby/issues/35888
+    return ' is not connected to the network ' in str(e)
+
+
 def is_conflict(e):
     conflict_infos = ['not running', 'not paused', 'paused']
     for info in conflict_infos:
@@ -96,6 +111,17 @@ class DockerDriver(driver.ContainerDriver):
     def __init__(self):
         super(DockerDriver, self).__init__()
         self._host = host.Host()
+        self._get_host_storage_info()
+        self.volume_driver = vol_driver.driver()
+        self.image_drivers = {}
+        for driver_name in CONF.image_driver_list:
+            driver = img_driver.load_image_driver(driver_name)
+            self.image_drivers[driver_name] = driver
+
+    def _get_host_storage_info(self):
+        storage_info = self._host.get_storage_info()
+        self.base_device_size = storage_info['default_base_size']
+        self.support_disk_quota = self._host.check_supported_disk_quota()
 
     def load_image(self, image_path=None):
         with docker_utils.docker_client() as docker:
@@ -114,15 +140,95 @@ class DockerDriver(driver.ContainerDriver):
         with docker_utils.docker_client() as docker:
             return docker.get_image(name)
 
-    def delete_image(self, image):
-        with docker_utils.docker_client() as docker:
-            LOG.debug('Deleting image %s', image)
-            return docker.remove_image(image)
+    def delete_image(self, context, img_id, image_driver=None):
+        image = self.inspect_image(img_id)['RepoTags'][0]
+        if image_driver:
+            image_driver_list = [image_driver.lower()]
+        else:
+            image_driver_list = CONF.image_driver_list
+        for driver_name in image_driver_list:
+            try:
+                image_driver = img_driver.load_image_driver(driver_name)
+                if driver_name == 'glance':
+                    image_driver.delete_image_tar(context, image)
+                elif driver_name == 'docker':
+                    image_driver.delete_image(context, img_id)
+            except exception.ZunException:
+                LOG.exception('Unknown exception occurred while deleting '
+                              'image %s', img_id)
+
+    def delete_committed_image(self, context, img_id, image_driver):
+        try:
+            image_driver.delete_committed_image(context, img_id)
+        except Exception as e:
+            LOG.exception('Unknown exception occurred while'
+                          'deleting image %s: %s',
+                          img_id,
+                          six.text_type(e))
+            raise exception.ZunException(six.text_type(e))
 
     def images(self, repo, quiet=False):
         with docker_utils.docker_client() as docker:
             return docker.images(repo, quiet)
 
+    def pull_image(self, context, repo, tag, image_pull_policy='always',
+                   driver_name=None):
+        if driver_name is None:
+            driver_name = CONF.default_image_driver
+
+        try:
+            image_driver = self.image_drivers[driver_name]
+            image, image_loaded = image_driver.pull_image(
+                context, repo, tag, image_pull_policy)
+            if image:
+                image['driver'] = driver_name.split('.')[0]
+        except Exception as e:
+            LOG.exception('Unknown exception occurred while loading '
+                          'image: %s', six.text_type(e))
+            raise exception.ZunException(six.text_type(e))
+
+        return image, image_loaded
+
+    def search_image(self, context, repo, tag, driver_name, exact_match):
+        if driver_name is None:
+            driver_name = CONF.default_image_driver
+
+        try:
+            image_driver = self.image_drivers[driver_name]
+            return image_driver.search_image(context, repo, tag,
+                                             exact_match)
+        except Exception as e:
+            LOG.exception('Unknown exception occurred while searching '
+                          'for image: %s', six.text_type(e))
+            raise exception.ZunException(six.text_type(e))
+
+    def create_image(self, context, image_name, image_driver):
+        try:
+            img = image_driver.create_image(context, image_name)
+        except Exception as e:
+            LOG.exception('Unknown exception occurred while creating '
+                          'image: %s', six.text_type(e))
+            raise exception.ZunException(six.text_type(e))
+        return img
+
+    def upload_image_data(self, context, image, image_tag, image_data,
+                          image_driver):
+        try:
+            image_driver.update_image(context,
+                                      image.id,
+                                      tag=image_tag)
+            # Image data has to match the image format.
+            # contain format defaults to 'docker';
+            # disk format defaults to 'qcow2'.
+            img = image_driver.upload_image_data(context,
+                                                 image.id,
+                                                 image_data)
+        except Exception as e:
+            LOG.exception('Unknown exception occurred while uploading '
+                          'image: %s', six.text_type(e))
+            raise exception.ZunException(six.text_type(e))
+        return img
+
     def read_tar_image(self, image):
         with docker_utils.docker_client() as docker:
             LOG.debug('Reading local tar image %s ', image['path'])
@@ -181,7 +287,7 @@ class DockerDriver(driver.ContainerDriver):
             if container.auto_remove:
                 host_config['auto_remove'] = container.auto_remove
             if container.memory is not None:
-                host_config['mem_limit'] = container.memory
+                host_config['mem_limit'] = str(container.memory) + 'M'
             if container.cpu is not None:
                 host_config['cpu_quota'] = int(100000 * container.cpu)
                 host_config['cpu_period'] = 100000
@@ -190,12 +296,16 @@ class DockerDriver(driver.ContainerDriver):
                 name = container.restart_policy['Name']
                 host_config['restart_policy'] = {'Name': name,
                                                  'MaximumRetryCount': count}
+
             if container.disk:
                 disk_size = str(container.disk) + 'G'
                 host_config['storage_opt'] = {'size': disk_size}
 
             kwargs['host_config'] = docker.create_host_config(**host_config)
-            image_repo = image['repo'] + ":" + image['tag']
+            if image['tag']:
+                image_repo = image['repo'] + ":" + image['tag']
+            else:
+                image_repo = image['repo']
             response = docker.create_container(image_repo, **kwargs)
             container.container_id = response['Id']
 
@@ -211,6 +321,12 @@ class DockerDriver(driver.ContainerDriver):
     def _is_runtime_supported(self):
         return float(CONF.docker.docker_remote_api_version) >= 1.26
 
+    def node_support_disk_quota(self):
+        return self.support_disk_quota
+
+    def get_host_default_base_size(self):
+        return self.base_device_size
+
     def _process_networking_config(self, context, container,
                                    requested_networks, host_config,
                                    container_kwargs, docker):
@@ -251,9 +367,8 @@ class DockerDriver(driver.ContainerDriver):
     def _get_binds(self, context, requested_volumes):
         binds = {}
         for volume in requested_volumes:
-            volume_driver = vol_driver.driver(provider=volume.volume_provider,
-                                              context=context)
-            source, destination = volume_driver.bind_mount(volume)
+            source, destination = self.volume_driver.bind_mount(context,
+                                                                volume)
             binds[source] = {'bind': destination}
         return binds
 
@@ -294,6 +409,8 @@ class DockerDriver(driver.ContainerDriver):
             except errors.APIError as api_error:
                 if is_not_found(api_error):
                     return
+                if is_not_connected(api_error):
+                    return
                 raise
 
     @wrap_docker_error
@@ -305,55 +422,100 @@ class DockerDriver(driver.ContainerDriver):
             network_api.disconnect_container_from_network(
                 container, docker_net, neutron_network_id=neutron_net)
 
+    def check_container_exist(self, container):
+        with docker_utils.docker_client() as docker:
+            docker_containers = [c['Id']
+                                 for c in docker.list_containers()]
+            if container.container_id not in docker_containers:
+                return False
+        return True
+
     def list(self, context):
         id_to_container_map = {}
+        uuids = []
         with docker_utils.docker_client() as docker:
+            docker_containers = docker.list_containers()
             id_to_container_map = {c['Id']: c
-                                   for c in docker.list_containers()}
+                                   for c in docker_containers}
+            uuids = self._get_container_uuids(docker_containers)
 
-        db_containers = objects.Container.list_by_host(context, CONF.host)
-        for db_container in db_containers:
-            if db_container.status in (consts.CREATING, consts.DELETING,
-                                       consts.DELETED):
+        local_containers = self._get_local_containers(context, uuids)
+        for container in local_containers:
+            if container.status in (consts.CREATING, consts.DELETING,
+                                    consts.DELETED):
                 # Skip populating db record since the container is in a
                 # unstable state.
                 continue
 
-            container_id = db_container.container_id
+            container_id = container.container_id
             docker_container = id_to_container_map.get(container_id)
             if not container_id or not docker_container:
-                if db_container.auto_remove:
-                    db_container.status = consts.DELETED
-                    db_container.save(context)
+                if container.auto_remove:
+                    container.status = consts.DELETED
+                    container.save(context)
                 else:
-                    LOG.warning("Container %s was recorded in DB but missing "
-                                "in docker", db_container.uuid)
+                    self.heal_with_rebuilding_container(context,
+                                                        container)
                 continue
 
-            self._populate_container(db_container, docker_container)
+            self._populate_container(container, docker_container)
 
-        return db_containers
+        return local_containers
+
+    def heal_with_rebuilding_container(self, context, container):
+        compute_api = zun_compute.API(context)
+        rebuild_status = [consts.CREATED, consts.RUNNING, consts.STOPPED]
+        try:
+            if (container.auto_heal and
+                    container.status in rebuild_status):
+                context.project_id = container.project_id
+                compute_api.container_rebuild(context, container)
+            else:
+                LOG.warning("Container %s was recorded in DB but "
+                            "missing in docker", container.uuid)
+                container.status = consts.ERROR
+                msg = "No such container:%s in docker" % \
+                      (container.container_id)
+                container.status_reason = six.text_type(msg)
+                container.save(context)
+        except Exception as e:
+            LOG.warning("heal container with rebuilding failed, "
+                        "err code: %s", e)
+
+    def _get_container_uuids(self, containers):
+        # The name of Docker container is of the form '/zun-<uuid>'
+        name_prefix = '/' + consts.NAME_PREFIX
+        uuids = [c['Names'][0].replace(name_prefix, '', 1)
+                 for c in containers]
+        return [u for u in uuids if uuidutils.is_uuid_like(u)]
+
+    def _get_local_containers(self, context, uuids):
+        host_containers = objects.Container.list_by_host(context, CONF.host)
+        uuids = list(set(uuids) | set([c.uuid for c in host_containers]))
+        containers = objects.Container.list(context,
+                                            filters={'uuid': uuids})
+        return containers
 
     def update_containers_states(self, context, containers):
-        db_containers = self.list(context)
-        if not db_containers:
+        local_containers = self.list(context)
+        if not local_containers:
             return
 
-        id_to_db_container_map = {container.container_id: container
-                                  for container in db_containers
-                                  if container.container_id}
+        id_to_local_container_map = {container.container_id: container
+                                     for container in local_containers
+                                     if container.container_id}
         id_to_container_map = {container.container_id: container
                                for container in containers
                                if container.container_id}
 
         for cid in (six.viewkeys(id_to_container_map) &
-                    six.viewkeys(id_to_db_container_map)):
+                    six.viewkeys(id_to_local_container_map)):
             container = id_to_container_map[cid]
             # sync status
-            db_container = id_to_db_container_map[cid]
-            if container.status != db_container.status:
+            local_container = id_to_local_container_map[cid]
+            if container.status != local_container.status:
                 old_status = container.status
-                container.status = db_container.status
+                container.status = local_container.status
                 container.save(context)
                 LOG.info('Status of container %s changed from %s to %s',
                          container.uuid, old_status, container.status)
@@ -428,12 +590,19 @@ class DockerDriver(driver.ContainerDriver):
 
     def _populate_container_state(self, container, state):
         if not state:
+            LOG.warning('Receive unexpected state from docker: %s', state)
             container.status = consts.UNKNOWN
+            container.status_reason = _("container state is missing")
             container.status_detail = None
         elif type(state) is dict:
             status_detail = ''
             if state.get('Error'):
-                container.status = consts.ERROR
+                if state.get('Status') in ('exited', 'removing'):
+                    container.status = consts.STOPPED
+                else:
+                    status = state.get('Status').capitalize()
+                    if status in consts.CONTAINER_STATUSES:
+                        container.status = status
                 status_detail = self.format_status_detail(
                     state.get('FinishedAt'))
                 container.status_detail = "Exited({}) {} ago " \
@@ -444,12 +613,18 @@ class DockerDriver(driver.ContainerDriver):
                     state.get('StartedAt'))
                 container.status_detail = "Up {} (paused)".format(
                     status_detail)
+            elif state.get('Restarting'):
+                container.status = consts.RESTARTING
+                container.status_detail = "Restarting"
             elif state.get('Running'):
                 container.status = consts.RUNNING
                 status_detail = self.format_status_detail(
                     state.get('StartedAt'))
                 container.status_detail = "Up {}".format(
                     status_detail)
+            elif state.get('Dead'):
+                container.status = consts.DEAD
+                container.status_detail = "Dead"
             else:
                 started_at = self.format_status_detail(state.get('StartedAt'))
                 finished_at = self.format_status_detail(
@@ -458,10 +633,14 @@ class DockerDriver(driver.ContainerDriver):
                     container.status = consts.CREATED
                     container.status_detail = "Created"
                 elif (started_at == "" and
-                        container.status in (consts.CREATED, consts.ERROR)):
+                        container.status in (consts.CREATED, consts.RESTARTING,
+                                             consts.ERROR, consts.REBUILDING)):
                     pass
                 elif started_at != "" and finished_at == "":
+                    LOG.warning('Receive unexpected state from docker: %s',
+                                state)
                     container.status = consts.UNKNOWN
+                    container.status_reason = _("unexpected container state")
                     container.status_detail = ""
                 elif started_at != "" and finished_at != "":
                     container.status = consts.STOPPED
@@ -474,26 +653,28 @@ class DockerDriver(driver.ContainerDriver):
             if state == 'created' and container.status == consts.CREATING:
                 container.status = consts.CREATED
             elif (state == 'created' and
-                    container.status in (consts.CREATED, consts.ERROR)):
+                    container.status in (consts.CREATED, consts.RESTARTING,
+                                         consts.ERROR, consts.REBUILDING)):
                 pass
             elif state == 'paused':
                 container.status = consts.PAUSED
             elif state == 'running':
                 container.status = consts.RUNNING
             elif state == 'dead':
-                container.status = consts.ERROR
-            elif state in ('restarting', 'exited', 'removing'):
+                container.status = consts.DEAD
+            elif state == 'restarting':
+                container.status = consts.RESTARTING
+            elif state in ('exited', 'removing'):
                 container.status = consts.STOPPED
             else:
+                LOG.warning('Receive unexpected state from docker: %s', state)
                 container.status = consts.UNKNOWN
+                container.status_reason = _("unexpected container state")
             container.status_detail = None
 
     def _populate_command(self, container, config):
         command_list = config.get('Cmd')
-        command_str = None
-        if command_list:
-            command_str = ' '.join(command_list)
-        container.command = command_str
+        container.command = command_list
 
     def _populate_hostname_and_ports(self, container, config):
         # populate hostname only when container.hostname wasn't set
@@ -606,10 +787,7 @@ class DockerDriver(driver.ContainerDriver):
                 raise exception.Conflict(_(
                     "Timeout on executing command: %s") % command)
             inspect_res = docker.exec_inspect(exec_id)
-            return {"output": output,
-                    "exit_code": inspect_res['ExitCode'],
-                    "exec_id": None,
-                    "url": None}
+            return output, inspect_res['ExitCode']
 
     def execute_resize(self, exec_id, height, width):
         height = int(height)
@@ -643,7 +821,7 @@ class DockerDriver(driver.ContainerDriver):
         args = {}
         memory = patch.get('memory')
         if memory is not None:
-            args['mem_limit'] = memory
+            args['mem_limit'] = str(memory) + 'M'
             args['memswap_limit'] = CONF.default_memory_swap
         cpu = patch.get('cpu')
         if cpu is not None:
@@ -688,7 +866,10 @@ class DockerDriver(driver.ContainerDriver):
             try:
                 stream, stat = docker.get_archive(
                     container.container_id, path)
-                filedata = stream.read(decode_content=True)
+                if isinstance(stream, types.GeneratorType):
+                    filedata = six.b("").join(stream)
+                else:
+                    filedata = stream.read()
                 return filedata, stat
             except errors.APIError as api_error:
                 if is_not_found(api_error):
@@ -784,7 +965,7 @@ class DockerDriver(driver.ContainerDriver):
             container.set_sandbox_id(sandbox['Id'])
             addresses = self._setup_network_for_container(
                 context, container, requested_networks, network_api)
-            if addresses is None:
+            if not addresses:
                 raise exception.ZunException(_(
                     "Unexpected missing of addresses"))
 
@@ -795,28 +976,16 @@ class DockerDriver(driver.ContainerDriver):
             return sandbox['Id']
 
     def attach_volume(self, context, volume_mapping):
-        volume_driver = vol_driver.driver(
-            provider=volume_mapping.volume_provider,
-            context=context)
-        volume_driver.attach(volume_mapping)
+        self.volume_driver.attach(context, volume_mapping)
 
     def detach_volume(self, context, volume_mapping):
-        volume_driver = vol_driver.driver(
-            provider=volume_mapping.volume_provider,
-            context=context)
-        volume_driver.detach(volume_mapping)
+        self.volume_driver.detach(context, volume_mapping)
 
     def delete_volume(self, context, volume_mapping):
-        volume_driver = vol_driver.driver(
-            provider=volume_mapping.volume_provider,
-            context=context)
-        volume_driver.delete(volume_mapping)
+        self.volume_driver.delete(context, volume_mapping)
 
     def is_volume_available(self, context, volume_mapping):
-        volume_driver = vol_driver.driver(
-            provider=volume_mapping.volume_provider,
-            context=context)
-        return volume_driver.is_volume_available(volume_mapping)
+        return self.volume_driver.is_volume_available(context, volume_mapping)
 
     def _get_or_create_docker_network(self, context, network_api,
                                       neutron_net_id):
@@ -850,10 +1019,10 @@ class DockerDriver(driver.ContainerDriver):
             docker.stop(sandbox_id)
 
     def get_sandbox_name(self, container):
-        return 'zun-sandbox-' + container.uuid
+        return consts.SANDBOX_NAME_PREFIX + container.uuid
 
     def get_container_name(self, container):
-        return 'zun-' + container.uuid
+        return consts.NAME_PREFIX + container.uuid
 
     def get_host_info(self):
         with docker_utils.docker_client() as docker:
@@ -877,6 +1046,18 @@ class DockerDriver(driver.ContainerDriver):
             return (total, running, paused, stopped, cpus,
                     architecture, os_type, os, kernel_version, labels)
 
+    def get_total_disk_for_container(self):
+        try:
+            (output, err) = utils.execute('df', '-B', '1G',
+                                          CONF.docker.docker_data_root,
+                                          run_as_root=True)
+        except exception.CommandError:
+            LOG.info('There was a problem while executing df -B 1G %s',
+                     CONF.docker.docker_data_root)
+            raise exception.CommandError(cmd='df')
+        total_disk = int(output.split('\n')[1].split()[1])
+        return int(total_disk * (1 - CONF.compute.reserve_disk_for_image))
+
     def get_cpu_used(self):
         cpu_used = 0
         with docker_utils.docker_client() as docker:
@@ -933,7 +1114,7 @@ class DockerDriver(driver.ContainerDriver):
             container.addresses = update
             container.save(context)
 
-    def network_attach(self, context, container, network):
+    def network_attach(self, context, container, requested_network):
         with docker_utils.docker_client() as docker:
             security_group_ids = None
             if container.security_groups:
@@ -941,18 +1122,14 @@ class DockerDriver(driver.ContainerDriver):
                     context, container.security_groups)
             network_api = zun_network.api(context,
                                           docker_api=docker)
+            network = requested_network['network']
             if network in container.addresses:
                 raise exception.ZunException('Container %(container)s has'
                                              ' alreay connected to the network'
                                              '%(network)s.'
-                                             % {'container': container['uuid'],
+                                             % {'container': container.uuid,
                                                 'network': network})
             self._get_or_create_docker_network(context, network_api, network)
-            requested_network = {'network': network,
-                                 'port': '',
-                                 'v4-fixed-ip': '',
-                                 'v6-fixed-ip': '',
-                                 'preserve_on_delete': False}
             docker_net_name = self._get_docker_network_name(context, network)
             addrs = network_api.connect_container_to_network(
                 container, docker_net_name, requested_network,
@@ -966,3 +1143,14 @@ class DockerDriver(driver.ContainerDriver):
             addresses.update(update)
             container.addresses = addresses
             container.save(context)
+
+    def create_network(self, context, network):
+        with docker_utils.docker_client() as docker:
+            network_api = zun_network.api(context,
+                                          docker_api=docker)
+            docker_net_name = self._get_docker_network_name(
+                context, network.neutron_net_id)
+            docker_network = network_api.create_network(
+                neutron_net_id=network.neutron_net_id,
+                name=docker_net_name)
+            return docker_network
