@@ -38,8 +38,8 @@ def load_container_driver(container_driver=None):
     if not container_driver:
         container_driver = CONF.container_driver
         if not container_driver:
-            LOG.error(("Container driver option required, "
-                       "but not specified"))
+            LOG.error("Container driver option required, "
+                      "but not specified")
             sys.exit(1)
 
     LOG.info("Loading container driver '%s'", container_driver)
@@ -58,13 +58,17 @@ def load_container_driver(container_driver=None):
 
 
 class ContainerDriver(object):
-    '''Base class for container drivers.'''
+    """Base class for container drivers."""
 
-    def create(self, context, container, sandbox_name=None):
+    def create(self, context, container, **kwargs):
         """Create a container."""
         raise NotImplementedError()
 
-    def delete(self, container, force):
+    def commit(self, context, container, repository, tag):
+        """Commit a container."""
+        raise NotImplementedError()
+
+    def delete(self, context, container, force):
         """Delete a container."""
         raise NotImplementedError()
 
@@ -76,36 +80,36 @@ class ContainerDriver(object):
         """Update containers states."""
         raise NotImplementedError()
 
-    def show(self, container):
+    def show(self, context, container):
         """Show the details of a container."""
         raise NotImplementedError()
 
-    def reboot(self, container):
+    def reboot(self, context, container):
         """Reboot a container."""
         raise NotImplementedError()
 
-    def stop(self, container):
+    def stop(self, context, container):
         """Stop a container."""
         raise NotImplementedError()
 
-    def start(self, container):
+    def start(self, context, container):
         """Start a container."""
         raise NotImplementedError()
 
-    def pause(self, container):
+    def pause(self, context, container):
         """Pause a container."""
         raise NotImplementedError()
 
-    def unpause(self, container):
-        """Pause a container."""
+    def unpause(self, context, container):
+        """Unpause a container."""
         raise NotImplementedError()
 
-    def show_logs(self, container, stdout=True, stderr=True,
+    def show_logs(self, context, container, stdout=True, stderr=True,
                   timestamps=False, tail='all', since=None):
         """Show logs of a container."""
         raise NotImplementedError()
 
-    def execute_create(self, container, command, **kwargs):
+    def execute_create(self, context, container, command, **kwargs):
         """Create an execute instance for running a command."""
         raise NotImplementedError()
 
@@ -117,35 +121,39 @@ class ContainerDriver(object):
         """Resizes the tty session used by the exec."""
         raise NotImplementedError()
 
-    def kill(self, container, signal):
-        """kill signal to a container."""
+    def kill(self, context, container, signal):
+        """Kill a container with specified signal."""
         raise NotImplementedError()
 
-    def get_websocket_url(self, container):
-        """get websocket url of a container."""
+    def get_websocket_url(self, context, container):
+        """Get websocket url of a container."""
         raise NotImplementedError()
 
-    def resize(self, container, height, weight):
-        """resize tty of a container."""
+    def resize(self, context, container, height, weight):
+        """Resize tty of a container."""
         raise NotImplementedError()
 
-    def top(self, container, ps_args):
-        """display the running processes inside the container."""
+    def top(self, context, container, ps_args):
+        """Display the running processes inside the container."""
         raise NotImplementedError()
 
-    def get_archive(self, container, path):
-        """copy resource froma container."""
+    def get_archive(self, context, container, path):
+        """Copy resource from a container."""
         raise NotImplementedError()
 
-    def put_archive(self, container, path, data):
-        """copy resource to a container."""
+    def put_archive(self, context, container, path, data):
+        """Copy resource to a container."""
         raise NotImplementedError()
 
-    def create_sandbox(self, context, container, **kwargs):
+    def stats(self, context, container):
+        """Display stats of the container."""
+        raise NotImplementedError()
+
+    def create_sandbox(self, context, *args, **kwargs):
         """Create a sandbox."""
         raise NotImplementedError()
 
-    def delete_sandbox(self, context, sandbox_id):
+    def delete_sandbox(self, context, container, sandbox_id):
         """Delete a sandbox."""
         raise NotImplementedError()
 
@@ -155,14 +163,6 @@ class ContainerDriver(object):
         """Stop a sandbox."""
         raise NotImplementedError()
 
-    def get_sandbox_id(self, container):
-        """Retrieve sandbox ID."""
-        raise NotImplementedError()
-
-    def set_sandbox_id(self, container, sandbox_id):
-        """Set sandbox ID."""
-        raise NotImplementedError()
-
     def get_sandbox_name(self, container):
         """Retrieve sandbox name."""
         raise NotImplementedError()
@@ -174,7 +174,7 @@ class ContainerDriver(object):
     def get_addresses(self, context, container):
         """Retrieve IP addresses of the container."""
 
-    def update(self, container):
+    def update(self, context, container):
         """Update a container."""
         raise NotImplementedError()
 
@@ -183,26 +183,55 @@ class ContainerDriver(object):
         os_capability_linux.LinuxHost().get_host_numa_topology(numa_topo_obj)
         return numa_topo_obj
 
+    def get_pci_resources(self):
+        return os_capability_linux.LinuxHost().get_pci_resources()
+
     def get_host_mem(self):
         return os_capability_linux.LinuxHost().get_host_mem()
 
     def get_host_info(self):
         raise NotImplementedError()
 
+    def get_total_disk_for_container(self):
+        return NotImplementedError()
+
     def get_cpu_used(self):
         raise NotImplementedError()
 
+    def attach_volume(self, context, volume_mapping):
+        raise NotImplementedError()
+
+    def detach_volume(self, context, volume_mapping):
+        raise NotImplementedError()
+
+    def delete_volume(self, context, volume_mapping):
+        raise NotImplementedError()
+
+    def is_volume_available(self, context, volume_mapping):
+        raise NotImplementedError()
+
+    def add_security_group(self, context, container, security_group, **kwargs):
+        raise NotImplementedError()
+
+    def remove_security_group(self, context, container, security_group,
+                              **kwargs):
+        raise NotImplementedError()
+
+    def get_available_nodes(self):
+        pass
+
     def get_available_resources(self, node):
         numa_topo_obj = self.get_host_numa_topology()
         node.numa_topology = numa_topo_obj
         meminfo = self.get_host_mem()
-        (mem_total, mem_free, mem_ava) = meminfo
+        (mem_total, mem_free, mem_ava, mem_used) = meminfo
         node.mem_total = mem_total // units.Ki
         node.mem_free = mem_free // units.Ki
         node.mem_available = mem_ava // units.Ki
+        node.mem_used = mem_used // units.Ki
         info = self.get_host_info()
         (total, running, paused, stopped, cpus,
-         architecture, os_type, os, kernel_version) = info
+         architecture, os_type, os, kernel_version, labels) = info
         node.total_containers = total
         node.running_containers = running
         node.paused_containers = paused
@@ -214,3 +243,51 @@ class ContainerDriver(object):
         node.kernel_version = kernel_version
         cpu_used = self.get_cpu_used()
         node.cpu_used = cpu_used
+        node.labels = labels
+        disk_total = self.get_total_disk_for_container()
+        node.disk_total = disk_total
+        disk_quota_supported = self.node_support_disk_quota()
+        node.disk_quota_supported = disk_quota_supported
+
+    def node_is_available(self, nodename):
+        """Return whether this compute service manages a particular node."""
+        if nodename in self.get_available_nodes():
+            return True
+        return False
+
+    def network_detach(self, context, container, network):
+        raise NotImplementedError()
+
+    def network_attach(self, context, container, requested_network):
+        raise NotImplementedError()
+
+    def create_network(self, context, network):
+        raise NotImplementedError()
+
+    def inspect_network(self, network):
+        raise NotImplementedError()
+
+    def node_support_disk_quota(self):
+        raise NotImplementedError()
+
+    def get_host_default_base_size(self):
+        raise NotImplementedError()
+
+    def pull_image(self, context, repo, tag, **kwargs):
+        raise NotImplementedError()
+
+    def search_image(self, context, repo, tag, driver_name, exact_match):
+        raise NotImplementedError()
+
+    def create_image(self, context, image_name, image_driver):
+        raise NotImplementedError()
+
+    def upload_image_data(self, context, image, image_tag, image_data,
+                          image_driver):
+        raise NotImplementedError()
+
+    def delete_committed_image(self, context, img_id, image_driver):
+        raise NotImplementedError()
+
+    def delete_image(self, context, img_id, image_driver):
+        raise NotImplementedError()
