@@ -17,15 +17,14 @@ Base API for Database
 
 from oslo_db import api as db_api
 
-from zun.common import exception
-from zun.common.i18n import _
 from zun.common import profiler
 import zun.conf
 
 """Add the database backend mapping here"""
 
 CONF = zun.conf.CONF
-_BACKEND_MAPPING = {'sqlalchemy': 'zun.db.sqlalchemy.api'}
+_BACKEND_MAPPING = {'sqlalchemy': 'zun.db.sqlalchemy.api',
+                    'etcd': 'zun.db.etcd.api'}
 IMPL = db_api.DBAPI.from_config(CONF,
                                 backend_mapping=_BACKEND_MAPPING,
                                 lazy=True)
@@ -34,15 +33,7 @@ IMPL = db_api.DBAPI.from_config(CONF,
 @profiler.trace("db")
 def _get_dbdriver_instance():
     """Return a DB API instance."""
-    if CONF.db_type == 'sql':
-        return IMPL
-    elif CONF.db_type == 'etcd':
-        import zun.db.etcd.api as etcd_api
-        return etcd_api.get_connection()
-    else:
-        raise exception.ConfigInvalid(
-            _("db_type value of %s is invalid, "
-              "must be sql or etcd") % CONF.db_type)
+    return IMPL
 
 
 @profiler.trace("db")
@@ -52,6 +43,7 @@ def list_containers(context, filters=None, limit=None, marker=None,
 
     Return a list of the specified columns for all containers that match
     the specified filters.
+
     :param context: The security context
     :param filters: Filters to apply. Defaults to None.
     :param limit: Maximum number of containers to return.
@@ -77,6 +69,7 @@ def create_container(context, values):
                    into the Drivers when managing this container. For
                    example:
                    ::
+
                     {
                      'uuid': uuidutils.generate_uuid(),
                      'name': 'example',
@@ -125,9 +118,9 @@ def destroy_container(context, container_id):
 def update_container(context, container_id, values):
     """Update properties of a container.
 
-    :context: Request context
+    :param context: Request context
     :param container_id: The id or uuid of a container.
-    :values: The properties to be updated
+    :param values: The properties to be updated
     :returns: A container.
     :raises: ContainerNotFound
     """
@@ -135,6 +128,76 @@ def update_container(context, container_id, values):
         context, container_id, values)
 
 
+@profiler.trace("db")
+def list_volume_mappings(context, filters=None, limit=None, marker=None,
+                         sort_key=None, sort_dir=None):
+    """List matching volume mappings.
+
+    Return a list of the specified columns for all volume mappings that match
+    the specified filters.
+
+    :param context: The security context
+    :param filters: Filters to apply. Defaults to None.
+    :param limit: Maximum number of containers to return.
+    :param marker: the last item of the previous page; we return the next
+                   result set.
+    :param sort_key: Attribute by which results should be sorted.
+    :param sort_dir: Direction in which results should be sorted.
+                     (asc, desc)
+    :returns: A list of tuples of the specified columns.
+    """
+    return _get_dbdriver_instance().list_volume_mappings(
+        context, filters, limit, marker, sort_key, sort_dir)
+
+
+@profiler.trace("db")
+def create_volume_mapping(context, values):
+    """Create a volume mapping.
+
+    :param context: The security context
+    :param values: A dict containing several items used to identify
+                   and track the volume mapping.
+    :returns: A volume mapping.
+    """
+    return _get_dbdriver_instance().create_volume_mapping(context, values)
+
+
+@profiler.trace("db")
+def get_volume_mapping_by_uuid(context, vm_uuid):
+    """Return a volume mapping.
+
+    :param context: The security context
+    :param vm_uuid: The uuid of a volume mapping.
+    :returns: A volume mapping.
+    """
+    return _get_dbdriver_instance().get_volume_mapping_by_uuid(
+        context, vm_uuid)
+
+
+@profiler.trace("db")
+def destroy_volume_mapping(context, vm_id):
+    """Destroy a volume mapping.
+
+    :param context: Request context
+    :param vm_id: The id or uuid of a volume mapping.
+    """
+    return _get_dbdriver_instance().destroy_volume_mapping(context, vm_id)
+
+
+@profiler.trace("db")
+def update_volume_mapping(context, vm_id, values):
+    """Update properties of a volume mapping.
+
+    :param context: Request context
+    :param vm_id: The id or uuid of a volume mapping.
+    :param values: The properties to be updated
+    :returns: A volume mapping.
+    :raises: VolumeMappingNotFound
+    """
+    return _get_dbdriver_instance().update_volume_mapping(
+        context, vm_id, values)
+
+
 @profiler.trace("db")
 def destroy_zun_service(host, binary):
     """Destroys a zun_service record.
@@ -190,7 +253,7 @@ def list_zun_services(context, filters=None, limit=None,
     those match the specified filters.
 
     :param context: The security context
-    :param disabled: Filters disbaled services. Defaults to None.
+    :param filters: Filters disbaled services. Defaults to None.
     :param limit: Maximum number of zun_services to return.
     :param marker: the last item of the previous page; we return the next
                    result set.
@@ -208,6 +271,7 @@ def list_zun_services_by_binary(context, binary):
     """List matching zun services.
 
     Return a list of the specified binary.
+
     :param context: The security context
     :param binary: The name of the binary.
     :returns: A list of tuples of the specified binary.
@@ -215,6 +279,12 @@ def list_zun_services_by_binary(context, binary):
     return _get_dbdriver_instance().list_zun_services_by_binary(binary)
 
 
+@profiler.trace("db")
+def destroy_image(context, image_uuid):
+    """Destroy a image"""
+    return _get_dbdriver_instance().destroy_image(context, image_uuid)
+
+
 @profiler.trace("db")
 def pull_image(context, values):
     """Create a new image.
@@ -226,6 +296,7 @@ def pull_image(context, values):
                    into the Drivers when managing this image. For
                    example:
                    ::
+
                     {
                      'uuid': uuidutils.generate_uuid(),
                      'repo': 'hello-world',
@@ -240,7 +311,8 @@ def pull_image(context, values):
 def update_image(image_id, values):
     """Update properties of an image.
 
-    :param container_id: The id or uuid of an image.
+    :param image_id: The id or uuid of an image.
+    :param values: The properties to be updated.
     :returns: An Image.
     :raises: ImageNotFound
     """
@@ -255,6 +327,7 @@ def list_images(context, filters=None,
 
     Return a list of the specified columns for all images that
     match the specified filters.
+
     :param context: The security context
     :param filters: Filters to apply. Defaults to None.
     :param limit: Maximum number of images to return.
@@ -298,6 +371,7 @@ def list_resource_providers(context, filters=None, limit=None, marker=None,
 
     Return a list of the specified columns for all resource providers that
     match the specified filters.
+
     :param context: The security context
     :param filters: Filters to apply. Defaults to None.
     :param limit: Maximum number of resource providers to return.
@@ -305,6 +379,8 @@ def list_resource_providers(context, filters=None, limit=None, marker=None,
                     return the next
     :param sort_key: Attribute by which results should be sorted.
                     (asc, desc)
+    :param sort_dir: Direction in which results should be sorted.
+                     (asc, desc)
     :returns: A list of tuples of the specified columns.
     """
     return _get_dbdriver_instance().list_resource_providers(
@@ -352,9 +428,9 @@ def destroy_resource_provider(context, provider_id):
 def update_resource_provider(context, provider_id, values):
     """Update properties of a resource provider.
 
-    :context: Request context
+    :param context: Request context
     :param provider_id: The id or uuid of a resource provider.
-    :values: The properties to be updated
+    :param values: The properties to be updated
     :returns: A resource provider.
     :raises: ResourceProviderNotFound
     """
@@ -368,6 +444,7 @@ def list_resource_classes(context, limit=None, marker=None, sort_key=None,
     """Get matching resource classes.
 
     Return a list of the specified columns for all resource classes.
+
     :param context: The security context
     :param limit: Maximum number of resource classes to return.
     :param marker: the last item of the previous page; we
@@ -421,9 +498,9 @@ def destroy_resource_class(context, resource_uuid):
 def update_resource_class(context, resource_uuid, values):
     """Update properties of a resource class.
 
-    :context: Request context
+    :param context: Request context
     :param resource_uuid: The uuid of a resource class.
-    :values: The properties to be updated
+    :param values: The properties to be updated
     :returns: A resource class.
     :raises: ResourceClassNotFound
     """
@@ -438,6 +515,7 @@ def list_inventories(context, filters=None, limit=None, marker=None,
 
     Return a list of the specified columns for all inventories that match
     the specified filters.
+
     :param context: The security context
     :param filters: Filters to apply. Defaults to None.
     :param limit: Maximum number of inventories to return.
@@ -493,9 +571,9 @@ def destroy_inventory(context, inventory_id):
 def update_inventory(context, inventory_id, values):
     """Update properties of an inventory.
 
-    :context: Request context
+    :param context: Request context
     :param inventory_id: The id of an inventory.
-    :values: The properties to be updated
+    :param values: The properties to be updated
     :returns: An inventory.
     :raises: InventoryNotFound
     """
@@ -510,6 +588,7 @@ def list_allocations(context, filters=None, limit=None, marker=None,
 
     Return a list of the specified columns for all allocations that match
     the specified filters.
+
     :param context: The security context
     :param filters: Filters to apply. Defaults to None.
     :param limit: Maximum number of allocations to return.
@@ -562,9 +641,9 @@ def destroy_allocation(context, allocation_id):
 def update_allocation(context, allocation_id, values):
     """Update properties of an allocation.
 
-    :context: Request context
+    :param context: Request context
     :param allocation_id: The id of an allocation.
-    :values: The properties to be updated
+    :param values: The properties to be updated
     :returns: An allocation.
     :raises: AllocationNotFound
     """
@@ -579,6 +658,7 @@ def list_compute_nodes(context, filters=None, limit=None, marker=None,
 
     Return a list of the specified columns for all compute nodes that match
     the specified filters.
+
     :param context: The security context
     :param filters: Filters to apply. Defaults to None.
     :param limit: Maximum number of compute nodes to return.
@@ -643,11 +723,344 @@ def destroy_compute_node(context, node_uuid):
 def update_compute_node(context, node_uuid, values):
     """Update properties of a compute node.
 
-    :context: Request context
+    :param context: Request context
     :param node_uuid: The uuid of a compute node.
-    :values: The properties to be updated
+    :param values: The properties to be updated
     :returns: A compute node.
     :raises: ComputeNodeNotFound
     """
     return _get_dbdriver_instance().update_compute_node(
         context, node_uuid, values)
+
+
+@profiler.trace("db")
+def list_capsules(context, filters=None, limit=None, marker=None,
+                  sort_key=None, sort_dir=None):
+    """List matching capsules.
+
+    Return a list of the specified columns for all capsules that match
+    the specified filters.
+
+    :param context: The security context
+    :param filters: Filters to apply. Defaults to None.
+    :param limit: Maximum number of capsules to return.
+    :param marker: the last item of the previous page; we return the next
+                   result set.
+    :param sort_key: Attribute by which results should be sorted.
+    :param sort_dir: Direction in which results should be sorted.
+                     (asc, desc)
+    :returns: A list of tuples of the specified columns.
+    """
+    return _get_dbdriver_instance().list_capsules(
+        context, filters, limit, marker, sort_key, sort_dir)
+
+
+@profiler.trace("db")
+def create_capsule(context, values):
+    """Create a new capsule.
+
+    :param context: The security context
+    :param values: A dict containing several items used to identify
+                   and track the container, and several dicts which are
+                   passed into the Drivers when managing this container.
+                   For example:
+                   ::
+
+                    {
+                     'uuid': uuidutils.generate_uuid(),
+                     'restart_policy': 'always',
+                     'project_id': '***'
+                    }
+    :returns: A capsule.
+    """
+    return _get_dbdriver_instance().create_capsule(context, values)
+
+
+@profiler.trace("db")
+def get_capsule_by_uuid(context, capsule_uuid):
+    """Return a capsule.
+
+    :param context: The security context
+    :param capsule_uuid: The uuid of a capsule.
+    :returns: A capsule.
+    """
+    return _get_dbdriver_instance().get_capsule_by_uuid(
+        context, capsule_uuid)
+
+
+@profiler.trace("db")
+def get_capsule_by_meta_name(context, capsule_name):
+    """Return a capsule.
+
+    :param context: The security context
+    :param capsule_name: The meta_name of a capsule.
+    :returns: A capsule.
+    """
+    return _get_dbdriver_instance().get_capsule_by_meta_name(
+        context, capsule_name)
+
+
+@profiler.trace("db")
+def destroy_capsule(context, capsule_id):
+    """Destroy a capsule and all associated interfaces.
+
+    :param context: Request context
+    :param capsule_id: The id or uuid of a capsule.
+    """
+    return _get_dbdriver_instance().destroy_capsule(context, capsule_id)
+
+
+@profiler.trace("db")
+def update_capsule(context, capsule_id, values):
+    """Update properties of a capsule.
+
+    :param context: Request context
+    :param capsule_id: The id or uuid of a capsule.
+    :param values: The properties to be updated
+    :returns: A capsule.
+    :raises: CapsuleNotFound
+    """
+    return _get_dbdriver_instance().update_capsule(
+        context, capsule_id, values)
+
+
+@profiler.trace("db")
+def get_pci_device_by_addr(node_id, dev_addr):
+    """Get PCI device by address."""
+    return _get_dbdriver_instance().get_pci_device_by_addr(node_id, dev_addr)
+
+
+@profiler.trace("db")
+def get_pci_device_by_id(id):
+    """Get PCI device by id."""
+    return _get_dbdriver_instance().get_pci_device_by_id(id)
+
+
+@profiler.trace("db")
+def get_all_pci_device_by_node(node_id):
+    """Get all PCI devices for one host."""
+    return _get_dbdriver_instance().get_all_pci_device_by_node(node_id)
+
+
+@profiler.trace("db")
+def get_all_pci_device_by_container_uuid(container_uuid):
+    """Get PCI devices allocated to container."""
+    return _get_dbdriver_instance().get_all_pci_device_by_container_uuid(
+        container_uuid)
+
+
+@profiler.trace("db")
+def get_all_pci_device_by_parent_addr(node_id, parent_addr):
+    """Get all PCI devices by parent address."""
+    return _get_dbdriver_instance().get_all_pci_device_by_parent_addr(
+        node_id, parent_addr)
+
+
+@profiler.trace("db")
+def destroy_pci_device(node_id, address):
+    """Delete a PCI device record."""
+    return _get_dbdriver_instance().destroy_pci_device(node_id, address)
+
+
+@profiler.trace("db")
+def update_pci_device(node_id, address, value):
+    """Update a pci device."""
+    return _get_dbdriver_instance().update_pci_device(node_id, address, value)
+
+
+@profiler.trace("db")
+def action_start(context, values):
+    """Start an action for an container."""
+    return _get_dbdriver_instance().action_start(context, values)
+
+
+@profiler.trace("db")
+def actions_get(context, uuid):
+    """Get all container actions for the provided container."""
+    return _get_dbdriver_instance().actions_get(context, uuid)
+
+
+@profiler.trace("db")
+def action_get_by_request_id(context, uuid, request_id):
+    """Get the action by request_id and given container."""
+    return _get_dbdriver_instance().action_get_by_request_id(context, uuid,
+                                                             request_id)
+
+
+@profiler.trace("db")
+def action_event_start(context, values):
+    """Start an event on an container action."""
+    return _get_dbdriver_instance().action_event_start(context, values)
+
+
+@profiler.trace("db")
+def action_event_finish(context, values):
+    """Finish an event on an container action."""
+    return _get_dbdriver_instance().action_event_finish(context, values)
+
+
+@profiler.trace("db")
+def action_events_get(context, action_id):
+    """Get the events by action id."""
+    return _get_dbdriver_instance().action_events_get(context, action_id)
+
+
+@profiler.trace("db")
+def quota_create(context, project_id, resource, limit):
+    """Create a quota for the given project and resource"""
+    return _get_dbdriver_instance().quota_create(context, project_id,
+                                                 resource, limit)
+
+
+@profiler.trace("db")
+def quota_get(context, project_id, resource):
+    """Retrieve a quota or raise if it does not exist"""
+    return _get_dbdriver_instance().quota_get(context, project_id,
+                                              resource)
+
+
+@profiler.trace("db")
+def quota_get_all_by_project(context, project_id):
+    """Retrieve all quotas associated with a given project"""
+    return _get_dbdriver_instance().quota_get_all_by_project(context,
+                                                             project_id)
+
+
+@profiler.trace("db")
+def quota_update(context, project_id, resource, limit):
+    """Update a quota or raise if it does not exist"""
+    return _get_dbdriver_instance().quota_update(context, project_id,
+                                                 resource, limit)
+
+
+@profiler.trace("db")
+def quota_destroy(context, project_id, resource):
+    """Destroy resource quota associated with a given project"""
+    return _get_dbdriver_instance().quota_destroy(context, project_id,
+                                                  resource)
+
+
+@profiler.trace("db")
+def quota_destroy_all_by_project(context, project_id, resource):
+    """Destroy all resource associated with a given project."""
+    return _get_dbdriver_instance().quota_destroy_all_by_project(context,
+                                                                 project_id)
+
+
+@profiler.trace("db")
+def quota_class_create(context, class_name, resource, limit):
+    """Create a quota class for the given name and resource"""
+    return _get_dbdriver_instance().quota_class_create(context, class_name,
+                                                       resource, limit)
+
+
+@profiler.trace("db")
+def quota_class_get(context, class_name, resource):
+    """Retrieve a quota class or raise if it does not exist"""
+    return _get_dbdriver_instance().quota_class_get(context, class_name,
+                                                    resource)
+
+
+@profiler.trace("db")
+def quota_class_get_default(context):
+    """Retrieve all default quota"""
+    return _get_dbdriver_instance().quota_class_get_default(context)
+
+
+@profiler.trace("db")
+def quota_class_get_all_by_name(context, class_name):
+    """Retrieve all quotas associated with a given quota class"""
+    return _get_dbdriver_instance().quota_class_get_all_by_name(
+        context, class_name)
+
+
+@profiler.trace("db")
+def quota_class_update(context, class_name, resource, limit):
+    """Update a quota class or raise if it does not exist"""
+    return _get_dbdriver_instance().quota_class_update(context, class_name,
+                                                       resource, limit)
+
+
+def quota_usage_get_all_by_project(context, project_id):
+    """Retrieve all usage associated with a given resource."""
+    return _get_dbdriver_instance().quota_usage_get_all_by_project(context,
+                                                                   project_id)
+
+
+@profiler.trace("db")
+def create_network(context, values):
+    """Create a new network.
+
+    :param context: The security context
+    :param values: A dict containing several items used to identify
+                   and track the network, and several dicts which are
+                   passed
+                   into the Drivers when managing this network. For
+                   example:
+                   ::
+                    {
+                     'uuid': uuidutils.generate_uuid(),
+                     'name': 'example',
+                     'type': 'virt'
+                    }
+    :returns: A network.
+    """
+    return _get_dbdriver_instance().create_network(context, values)
+
+
+@profiler.trace("db")
+def get_network_by_uuid(context, network_uuid):
+    """Return a network.
+
+    :param context: The security context
+    :param network_uuid: The uuid of a network.
+    :returns: A network.
+    """
+    return _get_dbdriver_instance().get_network_by_uuid(context, network_uuid)
+
+
+@profiler.trace("db")
+def update_network(context, uuid, values):
+    """Update properties of a network.
+
+    :param context: Request context
+    :param uuid: The id or uuid of a network.
+    :param values: The properties to be updated
+    :returns: A network.
+    """
+    return _get_dbdriver_instance().update_network(
+        context, uuid, values)
+
+
+@profiler.trace("db")
+def create_exec_instance(context, values):
+    """Create a new exec instance.
+
+    :param context: The security context
+    :param values: A dict containing several items used to identify
+                   and track the exec instance, and several dicts which are
+                   passed into the Drivers when managing this exec instance.
+    :returns: An exec instance.
+    """
+    return _get_dbdriver_instance().create_exec_instance(context, values)
+
+
+@profiler.trace("db")
+def list_exec_instances(context, filters=None, limit=None, marker=None,
+                        sort_key=None, sort_dir=None):
+    """List matching exec instances.
+
+    Return a list exec instances that match the specified filters.
+
+    :param context: The security context
+    :param filters: Filters to apply. Defaults to None.
+    :param limit: Maximum number of containers to return.
+    :param marker: the last item of the previous page; we return the next
+                   result set.
+    :param sort_key: Attribute by which results should be sorted.
+    :param sort_dir: Direction in which results should be sorted.
+                     (asc, desc)
+    :returns: A list of exec instances.
+    """
+    return _get_dbdriver_instance().list_exec_instances(
+        context, filters, limit, marker, sort_key, sort_dir)
