@@ -142,6 +142,8 @@ class KuryrNetwork(network.Network):
         admin_context = zun_context.get_admin_context()
         neutron_api = neutron.NeutronAPI(admin_context)
         subnetpool_id = subnet.get('subnetpool_id')
+        if not subnetpool_id:
+            return None
         if self._check_valid_subnetpool(neutron_api, subnetpool_id,
                                         subnet['cidr']):
             return subnetpool_id
@@ -157,7 +159,7 @@ class KuryrNetwork(network.Network):
             return None
         elif len(subnetpools) > 1:
             raise exception.ZunException(_(
-                'Multiple Neutron subnetpools exist with prefixes %s'),
+                'Multiple Neutron subnetpools exist with prefixes %s') %
                 subnet['cidr'])
         else:
             return subnetpools[0]['id']
@@ -170,7 +172,7 @@ class KuryrNetwork(network.Network):
             return subnets[0]
         else:
             raise exception.ZunException(_(
-                "Multiple Neutron subnets exist with ip version %s"),
+                "Multiple Neutron subnets exist with ip version %s") %
                 ip_version)
 
     def remove_network(self, network_name):
@@ -194,11 +196,10 @@ class KuryrNetwork(network.Network):
             # groups.
             if not container.security_groups:
                 container.security_groups = []
-            if neutron_port['security_groups']:
+            if neutron_port.get('security_groups'):
                 for sg in neutron_port['security_groups']:
                     if sg not in container.security_groups:
                         container.security_groups += [sg]
-                container.save(self.context)
 
             # update device_id in port
             port_req_body = {'port': {'device_id': container.uuid}}
@@ -228,8 +229,7 @@ class KuryrNetwork(network.Network):
                 'tenant_id': self.context.project_id,
                 'device_id': container.uuid,
             }
-            ip_addr = requested_network.get("v4-fixed-ip") or requested_network.\
-                get("v6-fixed-ip")
+            ip_addr = requested_network.get("fixed_ip")
             if ip_addr:
                 port_dict['fixed_ips'] = [{'ip_address': ip_addr}]
             if security_groups is not None:
@@ -247,6 +247,7 @@ class KuryrNetwork(network.Network):
                     'addr': ip_address,
                     'version': 4,
                     'port': neutron_port['id'],
+                    'subnet_id': fixed_ip['subnet_id'],
                     'preserve_on_delete': preserve_on_delete
                 })
             else:
@@ -254,6 +255,7 @@ class KuryrNetwork(network.Network):
                     'addr': ip_address,
                     'version': 6,
                     'port': neutron_port['id'],
+                    'subnet_id': fixed_ip['subnet_id'],
                     'preserve_on_delete': preserve_on_delete
                 })
 
