@@ -18,6 +18,9 @@ from zun.api import app
 from zun.tests.unit.api import base as api_base
 
 
+CURRENT_VERSION = api_base.CURRENT_VERSION
+
+
 class TestRootController(api_base.FunctionalTest):
     def setUp(self):
         super(TestRootController, self).setUp()
@@ -25,15 +28,16 @@ class TestRootController(api_base.FunctionalTest):
             'default_version':
             {'id': 'v1',
              'links': [{'href': 'http://localhost/v1/', 'rel': 'self'}],
-             'max_version': '1.4',
+             'max_version': '1.20',
              'min_version': '1.1',
              'status': 'CURRENT'},
             'description': 'Zun is an OpenStack project which '
-            'aims to provide container management.',
+            'aims to provide containers service.',
+            'name': 'OpenStack Zun API',
             'versions': [{'id': 'v1',
                           'links': [{'href': 'http://localhost/v1/',
                                      'rel': 'self'}],
-                          'max_version': '1.4',
+                          'max_version': '1.20',
                           'min_version': '1.1',
                           'status': 'CURRENT'}]}
 
@@ -60,10 +64,23 @@ class TestRootController(api_base.FunctionalTest):
                        'rel': 'self'},
                       {'href': 'http://localhost/hosts/',
                        'rel': 'bookmark'}],
+            'availability_zones': [
+                {'href': 'http://localhost/v1/availability_zones/',
+                 'rel': 'self'},
+                {'href': 'http://localhost/availability_zones/',
+                 'rel': 'bookmark'}],
             'images': [{'href': 'http://localhost/v1/images/',
                         'rel': 'self'},
                        {'href': 'http://localhost/images/',
-                        'rel': 'bookmark'}]}
+                        'rel': 'bookmark'}],
+            'networks': [{'href': 'http://localhost/v1/networks/',
+                          'rel': 'self'},
+                         {'href': 'http://localhost/networks/',
+                          'rel': 'bookmark'}],
+            'capsules': [{'href': 'http://localhost/v1/capsules/',
+                          'rel': 'self'},
+                         {'href': 'http://localhost/capsules/',
+                          'rel': 'bookmark'}]}
 
     def make_app(self, paste_file):
         file_name = self.get_path(paste_file)
@@ -85,15 +102,16 @@ class TestRootController(api_base.FunctionalTest):
     def test_noauth(self):
         # Don't need to auth
         paste_file = "zun/tests/unit/api/controllers/noauth-paste.ini"
+        headers = {'OpenStack-API-Version': CURRENT_VERSION}
         app = self.make_app(paste_file)
 
-        response = app.get('/')
+        response = app.get('/', headers=headers)
         self.assertEqual(self.root_expected, response.json)
 
-        response = app.get('/v1/')
+        response = app.get('/v1/', headers=headers)
         self.assertEqual(self.v1_expected, response.json)
 
-        response = app.get('/v1/containers/')
+        response = app.get('/v1/containers/', headers=headers)
         self.assertEqual(200, response.status_int)
 
     def test_auth_with_no_public_routes(self):
