@@ -14,13 +14,17 @@ from docker import errors
 import mock
 
 from oslo_utils import units
+from oslo_utils import uuidutils
 
 from zun.common import consts
+from zun.common import exception
 from zun import conf
 from zun.container.docker.driver import DockerDriver
 from zun.container.docker import utils as docker_utils
 from zun import objects
+from zun.objects.container import Container
 from zun.tests.unit.container import base
+from zun.tests.unit.db import utils
 from zun.tests.unit.objects import utils as obj_utils
 
 LSCPU_ON = """# The following is the parsable format, which can be fed to other
@@ -42,7 +46,10 @@ _numa_topo_spec = [_numa_node]
 
 
 class TestDockerDriver(base.DriverTestCase):
-    def setUp(self):
+
+    @mock.patch('zun.container.docker.driver.DockerDriver.'
+                '_get_host_storage_info')
+    def setUp(self, mock_get):
         super(TestDockerDriver, self).setUp()
         self.driver = DockerDriver()
         dfc_patcher = mock.patch.object(docker_utils, 'docker_client')
@@ -65,11 +72,14 @@ class TestDockerDriver(base.DriverTestCase):
         self.driver.get_image(name='image_name')
         self.mock_docker.get_image.assert_called_once_with('image_name')
 
-    def test_delete_image(self):
-        self.mock_docker.remove_image = mock.Mock()
-        mock_image = mock.MagicMock()
-        self.driver.delete_image(mock_image)
-        self.mock_docker.remove_image.assert_called_once_with(mock_image)
+    @mock.patch('zun.image.glance.driver.GlanceDriver.delete_image_tar')
+    def test_delete_image(self, mock_delete_image):
+        self.mock_docker.inspect_image = mock.Mock(
+            return_value={'RepoTags': ['ubuntu:1']})
+        img_id = '1234'
+        self.driver.delete_image(self.context, img_id, 'glance')
+        self.mock_docker.inspect_image.assert_called_once_with(img_id)
+        self.assertTrue(mock_delete_image.called)
 
     def test_load_image(self):
         self.mock_docker.load_image = mock.Mock()
@@ -90,10 +100,11 @@ class TestDockerDriver(base.DriverTestCase):
                 '.create_or_update_port')
     @mock.patch('zun.common.utils.get_security_group_ids')
     @mock.patch('zun.objects.container.Container.save')
-    def test_create_image_path_is_none(self, mock_save,
-                                       mock_get_security_group_ids,
-                                       mock_create_or_update_port,
-                                       mock_connect):
+    def test_create_image_path_is_none_with_overlay2(
+            self, mock_save,
+            mock_get_security_group_ids,
+            mock_create_or_update_port,
+            mock_connect):
         self.mock_docker.create_host_config = mock.Mock(
             return_value={'Id1': 'val1', 'key2': 'val2'})
         self.mock_docker.create_container = mock.Mock(
@@ -103,17 +114,20 @@ class TestDockerDriver(base.DriverTestCase):
         self.mock_docker.inspect_container = mock.Mock(
             return_value={'State': 'created',
                           'Config': {'Cmd': ['fake_command']}})
-        image = {'path': '', 'image': '', 'repo': '', 'tag': ''}
+        image = {'path': '', 'image': '', 'repo': 'test', 'tag': 'test'}
         mock_container = self.mock_default_container
         mock_container.status = 'Creating'
         networks = [{'network': 'fake-network'}]
         volumes = []
         fake_port = {'mac_address': 'fake_mac'}
         mock_create_or_update_port.return_value = ([], fake_port)
+        # DockerDriver with supported storage driver - overlay2
+        self.driver._host.sp_disk_quota = True
+        self.driver._host.storage_driver = 'overlay2'
         result_container = self.driver.create(self.context, mock_container,
                                               image, networks, volumes)
         host_config = {}
-        host_config['mem_limit'] = '512m'
+        host_config['mem_limit'] = '512M'
         host_config['cpu_quota'] = 100000
         host_config['cpu_period'] = 100000
         host_config['restart_policy'] = {'Name': 'no', 'MaximumRetryCount': 0}
@@ -125,8 +139,9 @@ class TestDockerDriver(base.DriverTestCase):
             **host_config)
 
         kwargs = {
-            'name': 'zun-ea8e2a25-2901-438d-8157-de7ffd68d051',
-            'command': 'fake_command',
+            'name': '%sea8e2a25-2901-438d-8157-de7ffd68d051' %
+                    consts.NAME_PREFIX,
+            'command': ['fake_command'],
             'environment': {'key1': 'val1', 'key2': 'val2'},
             'working_dir': '/home/ubuntu',
             'labels': {'key1': 'val1', 'key2': 'val2'},
@@ -144,10 +159,181 @@ class TestDockerDriver(base.DriverTestCase):
         self.assertEqual(result_container.status,
                          consts.CREATED)
 
-    def test_delete_success(self):
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.connect_container_to_network')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.create_or_update_port')
+    @mock.patch('zun.common.utils.get_security_group_ids')
+    @mock.patch('zun.objects.container.Container.save')
+    def test_create_image_path_is_none_with_devicemapper(
+            self, mock_save,
+            mock_get_security_group_ids,
+            mock_create_or_update_port,
+            mock_connect):
+        self.mock_docker.create_host_config = mock.Mock(
+            return_value={'Id1': 'val1', 'key2': 'val2'})
+        self.mock_docker.create_container = mock.Mock(
+            return_value={'Id': 'val1', 'key1': 'val2'})
+        self.mock_docker.create_networking_config = mock.Mock(
+            return_value={'Id': 'val1', 'key1': 'val2'})
+        self.mock_docker.inspect_container = mock.Mock(
+            return_value={'State': 'created',
+                          'Config': {'Cmd': ['fake_command']}})
+        image = {'path': '', 'image': '', 'repo': 'test', 'tag': 'test'}
+        mock_container = self.mock_default_container
+        mock_container.status = 'Creating'
+        networks = [{'network': 'fake-network'}]
+        volumes = []
+        fake_port = {'mac_address': 'fake_mac'}
+        mock_create_or_update_port.return_value = ([], fake_port)
+        # DockerDriver with supported storage driver - overlay2
+        self.driver._host.sp_disk_quota = True
+        self.driver._host.storage_driver = 'devicemapper'
+        self.driver._host.default_base_size = 10
+        result_container = self.driver.create(self.context, mock_container,
+                                              image, networks, volumes)
+        host_config = {}
+        host_config['mem_limit'] = '512M'
+        host_config['cpu_quota'] = 100000
+        host_config['cpu_period'] = 100000
+        host_config['restart_policy'] = {'Name': 'no', 'MaximumRetryCount': 0}
+        host_config['runtime'] = 'runc'
+        host_config['binds'] = {}
+        host_config['network_mode'] = 'fake-network'
+        host_config['storage_opt'] = {'size': '20G'}
+        self.mock_docker.create_host_config.assert_called_once_with(
+            **host_config)
+
+        kwargs = {
+            'name': '%sea8e2a25-2901-438d-8157-de7ffd68d051' %
+                    consts.NAME_PREFIX,
+            'command': ['fake_command'],
+            'environment': {'key1': 'val1', 'key2': 'val2'},
+            'working_dir': '/home/ubuntu',
+            'labels': {'key1': 'val1', 'key2': 'val2'},
+            'host_config': {'Id1': 'val1', 'key2': 'val2'},
+            'stdin_open': True,
+            'tty': True,
+            'hostname': 'testhost',
+            'volumes': [],
+            'networking_config': {'Id': 'val1', 'key1': 'val2'},
+            'mac_address': 'fake_mac',
+        }
+        self.mock_docker.create_container.assert_called_once_with(
+            image['repo'] + ":" + image['tag'], **kwargs)
+        self.assertEqual('val1', result_container.container_id)
+        self.assertEqual(result_container.status,
+                         consts.CREATED)
+
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.connect_container_to_network')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.create_or_update_port')
+    @mock.patch('zun.common.utils.get_security_group_ids')
+    @mock.patch('zun.objects.container.Container.save')
+    def test_create_docker_api_version_1_24(
+            self, mock_save,
+            mock_get_security_group_ids,
+            mock_create_or_update_port,
+            mock_connect):
+        CONF.set_override("docker_remote_api_version", "1.24", "docker")
+        self.mock_docker.create_host_config = mock.Mock(
+            return_value={'Id1': 'val1', 'key2': 'val2'})
+        self.mock_docker.create_container = mock.Mock(
+            return_value={'Id': 'val1', 'key1': 'val2'})
+        self.mock_docker.create_networking_config = mock.Mock(
+            return_value={'Id': 'val1', 'key1': 'val2'})
+        self.mock_docker.inspect_container = mock.Mock(
+            return_value={'State': 'created',
+                          'Config': {'Cmd': ['fake_command']}})
+        image = {'path': '', 'image': '', 'repo': 'test', 'tag': 'test'}
+        mock_container = self.mock_default_container
+        mock_container.status = 'Creating'
+        mock_container.runtime = None
+        networks = [{'network': 'fake-network'}]
+        volumes = []
+        fake_port = {'mac_address': 'fake_mac'}
+        mock_create_or_update_port.return_value = ([], fake_port)
+        result_container = self.driver.create(self.context, mock_container,
+                                              image, networks, volumes)
+        host_config = {}
+        host_config['mem_limit'] = '512M'
+        host_config['cpu_quota'] = 100000
+        host_config['cpu_period'] = 100000
+        host_config['restart_policy'] = {'Name': 'no', 'MaximumRetryCount': 0}
+        host_config['runtime'] = None
+        host_config['binds'] = {}
+        host_config['network_mode'] = 'fake-network'
+        host_config['storage_opt'] = {'size': '20G'}
+        self.mock_docker.create_host_config.assert_called_once_with(
+            **host_config)
+
+        kwargs = {
+            'name': '%sea8e2a25-2901-438d-8157-de7ffd68d051' %
+                    consts.NAME_PREFIX,
+            'command': ['fake_command'],
+            'environment': {'key1': 'val1', 'key2': 'val2'},
+            'working_dir': '/home/ubuntu',
+            'labels': {'key1': 'val1', 'key2': 'val2'},
+            'host_config': {'Id1': 'val1', 'key2': 'val2'},
+            'stdin_open': True,
+            'tty': True,
+            'hostname': 'testhost',
+            'volumes': [],
+            'networking_config': {'Id': 'val1', 'key1': 'val2'},
+            'mac_address': 'fake_mac',
+        }
+        self.mock_docker.create_container.assert_called_once_with(
+            image['repo'] + ":" + image['tag'], **kwargs)
+        self.assertEqual('val1', result_container.container_id)
+        self.assertEqual(result_container.status,
+                         consts.CREATED)
+
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.connect_container_to_network')
+    @mock.patch('zun.network.kuryr_network.KuryrNetwork'
+                '.create_or_update_port')
+    @mock.patch('zun.common.utils.get_security_group_ids')
+    @mock.patch('zun.objects.container.Container.save')
+    def test_create_docker_api_version_1_24_runtime_not_supported(
+            self, mock_save,
+            mock_get_security_group_ids,
+            mock_create_or_update_port,
+            mock_connect):
+        CONF.set_override("docker_remote_api_version", "1.24", "docker")
+        self.mock_docker.create_host_config = mock.Mock(
+            return_value={'Id1': 'val1', 'key2': 'val2'})
+        self.mock_docker.create_container = mock.Mock(
+            return_value={'Id': 'val1', 'key1': 'val2'})
+        self.mock_docker.create_networking_config = mock.Mock(
+            return_value={'Id': 'val1', 'key1': 'val2'})
+        self.mock_docker.inspect_container = mock.Mock(
+            return_value={'State': 'created',
+                          'Config': {'Cmd': ['fake_command']}})
+        image = {'path': '', 'image': '', 'repo': 'test', 'tag': 'test'}
+        mock_container = self.mock_default_container
+        mock_container.status = 'Creating'
+        mock_container.runtime = 'runc'
+        networks = [{'network': 'fake-network'}]
+        volumes = []
+        fake_port = {'mac_address': 'fake_mac'}
+        mock_create_or_update_port.return_value = ([], fake_port)
+        with self.assertRaisesRegex(
+                exception.ZunException,
+                "Specifying runtime in Docker API is not supported"):
+            self.driver.create(self.context, mock_container,
+                               image, networks, volumes)
+
+        self.mock_docker.create_host_config.assert_not_called()
+        self.mock_docker.create_container.assert_not_called()
+
+    @mock.patch('zun.container.docker.driver.DockerDriver'
+                '._cleanup_network_for_container')
+    def test_delete_success(self, mock_cleanup_network_for_container):
         self.mock_docker.remove_container = mock.Mock()
-        mock_container = mock.MagicMock()
+        mock_container = self.mock_default_container
         self.driver.delete(self.context, mock_container, True)
+        self.assertTrue(mock_cleanup_network_for_container.called)
         self.mock_docker.remove_container.assert_called_once_with(
             mock_container.container_id, force=True)
 
@@ -173,13 +359,91 @@ class TestDockerDriver(base.DriverTestCase):
                               True)
             self.mock_docker.remove_container.assert_called_once_with(
                 mock_container.container_id, force=True)
-            self.assertEqual(1, mock_init.call_count)
+            self.assertEqual(2, mock_init.call_count)
 
     def test_list(self):
         self.mock_docker.list_containers.return_value = []
         self.driver.list(self.context)
         self.mock_docker.list_containers.assert_called_once_with()
 
+    def test_get_container_uuids(self):
+        uuid = uuidutils.generate_uuid()
+        uuid2 = uuidutils.generate_uuid()
+        mock_container_list = [
+            {'Names': ['/%s%s' % (consts.NAME_PREFIX, uuid)]},
+            {'Names': ['/%s%s' % (consts.SANDBOX_NAME_PREFIX,
+                                  uuidutils.generate_uuid())]},
+            {'Names': ['/%s%s' % (consts.NAME_PREFIX, uuid2)]}]
+        uuids = self.driver._get_container_uuids(mock_container_list)
+        self.assertEqual(sorted([uuid, uuid2]), sorted(uuids))
+
+    @mock.patch('zun.objects.container.Container.list')
+    @mock.patch('zun.objects.container.Container.list_by_host')
+    def test_get_local_containers(self, mock_list_by_host, mock_list):
+        uuid = uuidutils.generate_uuid()
+        uuid2 = uuidutils.generate_uuid()
+        uuid3 = uuidutils.generate_uuid()
+        mock_container = obj_utils.get_test_container(
+            self.context, uuid=uuid, host='host')
+        mock_container_2 = obj_utils.get_test_container(
+            self.context, uuid=uuid2, host='host')
+        mock_container_3 = obj_utils.get_test_container(
+            self.context, uuid=uuid3, host='host2')
+
+        def fake_container_list(context, filters):
+            map = {}
+            map[uuid] = mock_container
+            map[uuid2] = mock_container_2
+            map[uuid3] = mock_container_3
+            return [map[u] for u in filters['uuid']]
+
+        def fake_container_list_by_host(context, host):
+            containers = [mock_container, mock_container_2, mock_container_3]
+            return [c for c in containers if c.host == conf.CONF.host]
+
+        mock_list.side_effect = fake_container_list
+        mock_list_by_host.side_effect = fake_container_list_by_host
+
+        # Containers in Docker matches DB records
+        conf.CONF.set_override('host', 'host')
+        docker_uuids = [uuid, uuid2]
+        local_containers = self.driver._get_local_containers(
+            self.context, docker_uuids)
+        self.assertEqual(2, len(local_containers))
+        self.assertIn(mock_container, local_containers)
+        self.assertIn(mock_container_2, local_containers)
+        self.assertNotIn(mock_container_3, local_containers)
+
+        # Containers in Docker doesn't match DB records
+        conf.CONF.set_override('host', 'host')
+        docker_uuids = [uuid2, uuid3]
+        local_containers = self.driver._get_local_containers(
+            self.context, docker_uuids)
+        self.assertEqual(3, len(local_containers))
+        self.assertIn(mock_container, local_containers)
+        self.assertIn(mock_container_2, local_containers)
+        self.assertIn(mock_container_3, local_containers)
+
+        # Containers are recorded in DB but missing in Docker
+        conf.CONF.set_override('host', 'host')
+        docker_uuids = []
+        local_containers = self.driver._get_local_containers(
+            self.context, docker_uuids)
+        self.assertEqual(2, len(local_containers))
+        self.assertIn(mock_container, local_containers)
+        self.assertIn(mock_container_2, local_containers)
+        self.assertNotIn(mock_container_3, local_containers)
+
+        # Containers are present in Docker but not recorded in DB
+        conf.CONF.set_override('host', 'host3')
+        docker_uuids = [uuid2, uuid3]
+        local_containers = self.driver._get_local_containers(
+            self.context, docker_uuids)
+        self.assertEqual(2, len(local_containers))
+        self.assertNotIn(mock_container, local_containers)
+        self.assertIn(mock_container_2, local_containers)
+        self.assertIn(mock_container_3, local_containers)
+
     @mock.patch('zun.objects.container.Container.save')
     def test_update_containers_states(self, mock_save):
         mock_container = obj_utils.get_test_container(
@@ -196,6 +460,24 @@ class TestDockerDriver(base.DriverTestCase):
             self.assertEqual(mock_container.host, 'host2')
             self.assertEqual(mock_container.status, 'Stopped')
 
+    @mock.patch('zun.compute.api.API.container_rebuild')
+    def test_heal_with_rebuilding_container(self, mock_container_rebuild):
+        mock_container = obj_utils.get_test_container(
+            self.context, status='Running',
+            auto_heal=True, task_state=None)
+        self.driver.heal_with_rebuilding_container(
+            self.context, mock_container)
+        mock_container_rebuild.assert_called_once_with(self.context,
+                                                       mock_container)
+
+    @mock.patch('zun.compute.api.API.container_rebuild')
+    def test_heal_with_rebuilding_exception(self, mock_container_rebuild):
+        container = Container(self.context, **utils.get_test_container())
+        container.status = consts.RUNNING
+        mock_container_rebuild.side_effect = Exception
+        self.driver.heal_with_rebuilding_container(
+            self.context, container)
+
     def test_show_success(self):
         self.mock_docker.inspect_container = mock.Mock(
             return_value={'State': 'running'})
@@ -212,7 +494,7 @@ class TestDockerDriver(base.DriverTestCase):
         self.driver.show(self.context, mock_container)
         self.mock_docker.inspect_container.assert_called_once_with(
             mock_container.container_id)
-        self.assertEqual('fake_command', mock_container.command)
+        self.assertEqual(['fake_command'], mock_container.command)
 
     def test_show_without_command(self):
         self.mock_docker.inspect_container = mock.Mock(
@@ -265,6 +547,40 @@ class TestDockerDriver(base.DriverTestCase):
                              consts.ERROR)
             self.assertEqual(2, mock_init.call_count)
 
+    def test_show_container_status_error_stop(self):
+        self.mock_docker.inspect_container = mock.Mock(
+            return_value={'State': {
+                'Status': 'exited',
+                'Error': 'Container start error.',
+                'FinishedAt': '0001-01-01T00:00:00Z',
+            }}
+        )
+        mock_container = mock.MagicMock()
+        mock_container.status = 'existed'
+        mock_container.Error = 'Container start error.'
+        self.driver.show(self.context, mock_container)
+        self.mock_docker.inspect_container.assert_called_once_with(
+            mock_container.container_id
+        )
+        self.assertEqual(consts.STOPPED, mock_container.status)
+
+    def test_show_container_status_error_unknown(self):
+        self.mock_docker.inspect_container = mock.Mock(
+            return_value={'State': {
+                'Status': 'Unknown',
+                'Error': 'Container run error.',
+                'FinishedAt': '0001-01-01T00:00:00Z',
+            }}
+        )
+        mock_container = mock.MagicMock()
+        mock_container.status = 'UNKNOWN'
+        mock_container.Error = 'Container run error.'
+        self.driver.show(self.context, mock_container)
+        self.mock_docker.inspect_container.assert_called_once_with(
+            mock_container.container_id
+        )
+        self.assertEqual(consts.UNKNOWN, mock_container.status)
+
     def test_show_status_deleting(self):
         with mock.patch.object(errors.APIError, '__str__',
                                return_value='404 Not Found') as mock_init:
@@ -476,16 +792,19 @@ class TestDockerDriver(base.DriverTestCase):
         mock_container = mock.MagicMock(
             uuid='ea8e2a25-2901-438d-8157-de7ffd68d051')
         result_sanbox_name = self.driver.get_sandbox_name(mock_container)
-        self.assertEqual(result_sanbox_name,
-                         'zun-sandbox-ea8e2a25-2901-438d-8157-de7ffd68d051')
+        self.assertEqual(
+            result_sanbox_name,
+            '%sea8e2a25-2901-438d-8157-de7ffd68d051' %
+            consts.SANDBOX_NAME_PREFIX)
 
     def test_get_container_name(self):
         mock_container = mock.MagicMock(
             uuid='ea8e2a25-2901-438d-8157-de7ffd68d051')
         result_container_name = self.driver.get_container_name(
             mock_container)
-        self.assertEqual(result_container_name,
-                         'zun-ea8e2a25-2901-438d-8157-de7ffd68d051')
+        self.assertEqual(
+            result_container_name,
+            '%sea8e2a25-2901-438d-8157-de7ffd68d051' % consts.NAME_PREFIX)
 
     def test_execute_resize(self):
         self.mock_docker.exec_resize = mock.Mock()
@@ -551,7 +870,7 @@ class TestDockerDriver(base.DriverTestCase):
             'memory_stats': {'usage': 104857600,
                              'limit': 1048576000},
             'networks': {'eth0':
-                            {'tx_dropped': 0, 'rx_packets': 2, 'rx_bytes': 200,
+                         {'tx_dropped': 0, 'rx_packets': 2, 'rx_bytes': 200,
                              'tx_errors': 0, 'rx_errors': 0, 'tx_bytes': 200,
                              'rx_dropped': 0, 'tx_packets': 2}}}
         stats_info = self.driver.stats(self.context, mock_container)
@@ -578,20 +897,33 @@ class TestDockerDriver(base.DriverTestCase):
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
                 '.list_networks')
     def test_network_attach(self, mock_list, mock_disconnect, mock_connect):
-        mock_container = mock.MagicMock()
+        mock_container = mock.Mock()
         mock_container.security_groups = None
+        mock_container.addresses = {}
         mock_list.return_value = {'network': 'network'}
-        requested_network = [{'network': 'network',
-                              'port': '',
-                              'v4-fixed-ip': '',
-                              'v6-fixed-ip': '',
-                              'preserve_on_delete': False}]
-        self.driver.network_attach(self.context, mock_container, 'network')
+        requested_network = {'network': 'network',
+                             'port': '',
+                             'fixed_ip': '',
+                             'preserve_on_delete': False}
+        self.driver.network_attach(self.context, mock_container,
+                                   requested_network)
         mock_connect.assert_called_once_with(mock_container,
                                              'network',
-                                             requested_network[0],
+                                             requested_network,
                                              security_groups=None)
 
+    def test_network_attach_error(self):
+        mock_container = mock.Mock()
+        mock_container.security_groups = None
+        mock_container.addresses = {'already-attached-net': []}
+        requested_network = {'network': 'already-attached-net',
+                             'port': '',
+                             'fixed_ip': '',
+                             'preserve_on_delete': False}
+        self.assertRaises(exception.ZunException,
+                          self.driver.network_attach,
+                          self.context, mock_container, requested_network)
+
     @mock.patch('zun.common.utils.get_security_group_ids')
     @mock.patch('zun.network.kuryr_network.KuryrNetwork'
                 '.connect_container_to_network')
@@ -601,29 +933,32 @@ class TestDockerDriver(base.DriverTestCase):
                                                 mock_connect,
                                                 mock_get_sec_group_id):
         test_sec_group_id = '84e3a4c1-c8cd-46b1-a0d9-c8c35f6a32a4'
-        mock_container = mock.MagicMock()
+        mock_container = mock.Mock()
         mock_container.security_groups = ['test_sec_group']
+        mock_container.addresses = {}
         mock_list.return_value = {'network': 'network'}
         mock_get_sec_group_id.return_value = test_sec_group_id
-        requested_network = [{'network': 'network',
-                              'port': '',
-                              'v4-fixed-ip': '',
-                              'v6-fixed-ip': '',
-                              'preserve_on_delete': False}]
-        self.driver.network_attach(self.context, mock_container, 'network')
+        requested_network = {'network': 'network',
+                             'port': '',
+                             'fixed_ip': '',
+                             'preserve_on_delete': False}
+        self.driver.network_attach(self.context, mock_container,
+                                   requested_network)
         mock_connect.assert_called_once_with(mock_container,
                                              'network',
-                                             requested_network[0],
+                                             requested_network,
                                              security_groups=test_sec_group_id)
 
     @mock.patch('zun.common.utils.execute')
+    @mock.patch('zun.container.docker.driver.DockerDriver'
+                '.get_total_disk_for_container')
     @mock.patch('zun.container.driver.ContainerDriver.get_host_mem')
     @mock.patch(
         'zun.container.docker.driver.DockerDriver.get_host_info')
     @mock.patch(
         'zun.container.docker.driver.DockerDriver.get_cpu_used')
     def test_get_available_resources(self, mock_cpu_used, mock_info, mock_mem,
-                                     mock_output):
+                                     mock_disk, mock_output):
         self.driver = DockerDriver()
         mock_output.return_value = LSCPU_ON
         conf.CONF.set_override('floating_cpu_set', "0")
@@ -633,6 +968,7 @@ class TestDockerDriver(base.DriverTestCase):
                                   'CentOS', '3.10.0-123',
                                   {'dev.type': 'product'})
         mock_cpu_used.return_value = 1.0
+        mock_disk.return_value = 80
         node_obj = objects.ComputeNode()
         self.driver.get_available_resources(node_obj)
         self.assertEqual(_numa_topo_spec, node_obj.numa_topology.to_list())
@@ -650,3 +986,4 @@ class TestDockerDriver(base.DriverTestCase):
         self.assertEqual('CentOS', node_obj.os)
         self.assertEqual('3.10.0-123', node_obj.kernel_version)
         self.assertEqual({'dev.type': 'product'}, node_obj.labels)
+        self.assertEqual(80, node_obj.disk_total)
