@@ -344,24 +344,28 @@ class TestObject(test_base.TestCase, _TestObject):
 # For more information on object version testing, read
 # https://docs.openstack.org/zun/latest/
 object_data = {
-    'Container': '1.24-bd8b0f179b62157772955f0a76735a39',
+    'Container': '1.33-5eac0a995f25329ca566fdddde45c759',
     'VolumeMapping': '1.1-50df6202f7846a136a91444c38eba841',
-    'Image': '1.0-0b976be24f4f6ee0d526e5c981ce0633',
+    'Image': '1.1-330e6205c80b99b59717e1cfc6a79935',
     'MyObj': '1.0-34c4b1aadefd177b13f9a2f894cc23cd',
     'NUMANode': '1.0-cba878b70b2f8b52f1e031b41ac13b4e',
     'NUMATopology': '1.0-b54086eda7e4b2e6145ecb6ee2c925ab',
     'ResourceClass': '1.1-d661c7675b3cd5b8c3618b68ba64324e',
     'ResourceProvider': '1.0-92b427359d5a4cf9ec6c72cbe630ee24',
-    'ZunService': '1.1-b1549134bfd5271daec417ca8cabc77e',
-    'Capsule': '1.5-cbdaffa78fa68c26cf4a61d8f75dd32d',
+    'ZunService': '1.2-deff2a74a9ce23baa231ae12f39a6189',
+    'Capsule': '1.6-7238a80b4ef34e219c135fa72d0adc06',
     'PciDevice': '1.1-6e3f0851ad1cf12583e6af4df1883979',
-    'ComputeNode': '1.9-e8536102d3b28cb3378e9e26f508cd72',
+    'ComputeNode': '1.11-08be22db017745f4f0bc8f873eca7db0',
     'PciDevicePool': '1.0-3f5ddc3ff7bfa14da7f6c7e9904cc000',
     'PciDevicePoolList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
+    'Quota': '1.2-3a7d520d119fe1e886baad968ef7990a',
+    'QuotaClass': '1.1-239ae335b32036b86504684d3fdbeb7f',
     'ContainerPCIRequest': '1.0-b060f9f9f734bedde79a71a4d3112ee0',
     'ContainerPCIRequests': '1.0-7b8f7f044661fe4e24e6949c035af2c4',
     'ContainerAction': '1.1-b0c721f9e10c6c0d1e41e512c49eb877',
-    'ContainerActionEvent': '1.0-2974d0a6f5d4821fd4e223a88c10181a'
+    'ContainerActionEvent': '1.0-2974d0a6f5d4821fd4e223a88c10181a',
+    'Network': '1.0-235ba13359282107f27c251af9aaffcd',
+    'ExecInstance': '1.0-59464e7b96db847c0abb1e96d3cec30a',
 }
 
 
